package fr.ensicaen.ecole.easygame;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

import fr.ensicaen.ecole.easygame.messaging.AfterTurnSubmitMessage;
import fr.ensicaen.ecole.easygame.messaging.BeforeTurnMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.utils.NicknameGenerator;

public final class AIClient extends Thread implements Client {

	private Game _game;
	private MessageReceiver _logic;
	private String _address;
	private int _port;
	private AILogic _aiLogic;
	private String _wantedNickname;
	private ClassLoader _loader;

	public AIClient(String address, int port, LaunchableGame lgame, ClassLoader loader, AILogic aiLogic) {
		_game = new Game(lgame, false, loader);
		_logic = lgame.instantiateLogic();
		_aiLogic = aiLogic;
		_address = address;
		_port = port;
		_wantedNickname = NicknameGenerator.generate();
		_loader = loader;
	}

	public void run() {
		_logic.link(_game);

		try {
			Socket socket = new Socket(_address, _port);
			ObjectInputStream streamInput;

			if (_loader != null) {
				streamInput = new SmarterObjectInputStream(socket.getInputStream(), _loader);
			} else {
				streamInput = new ObjectInputStream(socket.getInputStream());
			}
			ObjectOutputStream streamOutput = new ObjectOutputStream(socket.getOutputStream());

			streamOutput.writeObject(_wantedNickname);
			int id = (Integer) streamInput.readObject();

			while (true) {
				GameState gs = (GameState) streamInput.readObject();
				_game.setGameState(gs);

				if (_game.isEndOfGame()) {
					break;
				}

				if (id == _game.getGameState().getCurrentPlayer().getId()) {
					MessageReceiver.receive(new BeforeTurnMessage());

					if (!_game.isEndOfTurn()) {
						_aiLogic.think(_game, _game.getGameState().getCurrentPlayer());
					}

					MessageReceiver.receive(new AfterTurnSubmitMessage());
					_game.nextTurn();
					streamOutput.writeObject(_game.getGameState());
				}
			}
			streamInput.close();
			streamOutput.close();
			socket.close();
		} catch (UnknownHostException | ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IOException ignored) {
		}
	}

	@Override
	public void registerPlugins(LaunchableGame lg) {
	}
}
