package fr.ensicaen.ecole.easygame;

import javax.swing.JFrame;
import javax.swing.UIManager;

public final class MiniManager {

	private MiniManager() {
	}

	private static void host(LaunchableGame lg, int port, int numberOfClientsExpected) {
		Server s = new Server(port, lg, numberOfClientsExpected, null);
		if (s.tryBind()) {
			s.start();
		}
	}

	private static void human(LaunchableGame lg, String ip, int port) {
		JFrame window = new JFrame();
		HumanClient c = new HumanClient(ip, port, lg, window, null, "P");

		c.registerPlugins(lg);
		c.tryConnect();
		c.start();
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setVisible(true);
	}

	private static void nHumans(LaunchableGame lg, String ip, int port, int n) {
		JFrame window = new JFrame();

		for (int i = 0; i < n; i++) {
			HumanClient c = new HumanClient(ip, port, lg, window, null, "P" + i);

			c.registerPlugins(lg);
			c.tryConnect();
			c.start();
			c.setShared();
		}

		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setVisible(true);
	}

	private static void ai(LaunchableGame lg, String ip, int port) throws InstantiationException, IllegalAccessException {
		AILogic ai = lg.getAvailableAIs().get(0).newInstance();
		Client c = new AIClient(ip, port, lg, null, ai);

		c.start();
	}

	/**
	 * Runs a game.
	 *
	 * @param className Main class of the game. (Extending LaunchableGame)
	 * @param nbPlayers Number of players.
	 */
	public static void run(String className, int nbPlayers) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		try {
			LaunchableGame lg = (LaunchableGame) Class.forName(className).newInstance();

			host(lg, 4545, nbPlayers);

			if (lg.getAvailableAIs().size() >= 1) {
				for (int i = 0; i < nbPlayers - 1; i++) {
					human(lg, "localhost", 4545);
				}

				ai(lg, "localhost", 4545);
			} else {
				nHumans(lg, "localhost", 4545, nbPlayers);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
}
