package fr.ensicaen.ecole.easygame;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;

import fr.ensicaen.ecole.easygame.messaging.AfterTurnMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.GameState;

public final class Server extends Thread {

	private ServerSocket _socket;
	private Game _game;
	private MessageReceiver _logic;
	private List<ServerClient> _clientList;
	private int _port;
	private int _numberOfClientExpected;
	private ClassLoader _loader;

	public Server(int port, LaunchableGame lgame, int numberOfClientsExpected, ClassLoader loader) {
		_game = new Game(lgame, false, loader);
		_logic = lgame.instantiateLogic();
		_port = port;
		_numberOfClientExpected = numberOfClientsExpected;
		_loader = loader;

		_game.setGameState(lgame.instantiateGameState());
		_clientList = new ArrayList<>();
	}

	public boolean tryBind() {
		try {
			_socket = new ServerSocket(_port);
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public void run() {
		_logic.link(_game);

		try {
			while (_clientList.size() != _numberOfClientExpected) {
				Socket socket = _socket.accept();
				ServerClient c = new ServerClient(socket, _clientList.size(), _loader);
				c.send(c.getId());

				// detect duplicates
				String nick = (String) c.receiveObject();
				do {
					for (ServerClient c2 : _clientList) {
						if (c2.getNickname().equals(nick)) {
							nick = nick + '_';
						}
					}
				} while (false);
				c.setNickname(nick);

				_clientList.add(c);
			}

			MessageReceiver.receive(new PrepareMessage(_clientList));
			_game.run();

			for (ServerClient c : _clientList) {
				c.send(_game.getGameState());
			}

			while (!_game.isEndOfGame()) {
				GameState gs = (GameState) _clientList.get(_game.getGameState().getCurrentPlayer().getId()).receiveObject();
				_game.setGameState(gs);
				MessageReceiver.receive(new AfterTurnMessage());

				for (ServerClient c : _clientList) {
					c.send(gs);
				}
			}

			_socket.close();
		} catch (IOException e) {
			for (ServerClient c : _clientList) {
				try {
					c.disconnect();
				} catch (IOException ignored) {
				}
			}

			try {
				if (_socket != null) {
					_socket.close();
				}
			} catch (IOException e2) {
				e2.printStackTrace();
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
}
