package fr.ensicaen.ecole.easygame;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.Socket;

public final class ServerClient {

	private int _id;
	private String _nickname;
	private Socket _socket;
	private ObjectOutputStream _output;
	private ObjectInputStream _input;
	private ClassLoader _loader;

	ServerClient(Socket socket, int id, ClassLoader loader) throws IOException {
		_loader = loader;
		_socket = socket;
		_id = id;
		_output = new ObjectOutputStream(socket.getOutputStream());

		if (loader != null) {
			_input = new SmarterObjectInputStream(socket.getInputStream(), _loader);
		} else {
			_input = new ObjectInputStream(socket.getInputStream());
		}
	}

	void disconnect() throws IOException {
		_socket.close();
	}

	void send(Serializable object) throws IOException {
		_output.writeObject(object);
	}

	public int getId() {
		return _id;
	}

	Object receiveObject() throws ClassNotFoundException, IOException {
		return _input.readObject();
	}

	void setNickname(String nickname) {
		_nickname = nickname;
	}

	public String getNickname() {
		return _nickname;
	}
}
