package fr.ensicaen.ecole.easygame.dialogs;

import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A dialog box that announces the end of a game with custom text.
 * Used for example to announce a winner.
 */
public final class Dialog extends JDialog {

	private static final long serialVersionUID = 1L;

	private List<JPanel> _linePanels = new ArrayList<>();

	/**
	 * @param title Title of the window
	 * @param message First line of text that has to be printed.
	 */
	public Dialog(String title, String message) {
		setTitle(title);
		setModal(true);

		if (!message.isEmpty()) {
			JPanel jpTmp = new JPanel();
			jpTmp.add(new JLabel(message));
			_linePanels.add(jpTmp);
		}
	}

	/**
	 * Adds a line to be printed in the window with a label.
	 *
	 * @param name Text for the label.
	 * @param info Text to be printed.
	 */
	public void addLine(String name, String info) {
		JPanel jpTmp = new JPanel(new GridLayout(1, 2));
		jpTmp.add(new JLabel(name));
		jpTmp.add(new JLabel(info));
		_linePanels.add(jpTmp);
	}

	/**
	 * Displays the window.
	 */
	public void display() {
		JPanel jpanel = new JPanel(new GridLayout(_linePanels.size(), 1));

		for (JPanel p : _linePanels) {
			jpanel.add(p);
		}

		add(jpanel);
		pack();
		setSize(getWidth() + 50, getHeight() + 10);
		setLocationRelativeTo(null);
		setVisible(true);
	}
}
