package fr.ensicaen.ecole.easygame.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import fr.ensicaen.ecole.easygame.Game;

/**
 * GUI displaying the game, the inventories and the plugins.
 */
public final class GamePane extends JPanel {

	private static final long serialVersionUID = 1L;

	private StatusColumn _statusColumn;
	private GameZone _gameZone;
	private JScrollPane _gameScroll;
	private InventoryZone _inventoryZone;

	public GamePane(Game game) {
		setPreferredSize(new Dimension(800, 600));

		_statusColumn = new StatusColumn(game);
		_gameZone = new GameZone(game);
		_inventoryZone = new InventoryZone(game);
		setLayout(new BorderLayout());

		JPanel rightSidePanel = new JPanel();
		add(_statusColumn, BorderLayout.LINE_START);
		add(rightSidePanel, BorderLayout.CENTER);

		_gameScroll = new JScrollPane(_gameZone);

		rightSidePanel.setLayout(new BorderLayout());
		rightSidePanel.add(_inventoryZone, BorderLayout.PAGE_END);
		rightSidePanel.add(_gameScroll, BorderLayout.CENTER);
	}

	/**
	 * @return The GUI displaying the status column.
	 */
	public StatusColumn getStatusColumn() {
		return _statusColumn;
	}

	/**
	 * @return The GUI displaying the game board.
	 */
	public GameZone getGameZone() {
		return _gameZone;
	}

	/**
	 * @return The GUI displaying the inventories.
	 */
	public InventoryZone getInventoryZone() {
		return _inventoryZone;
	}

	/**
	 * Centers the zone displaying the game board.
	 */
	public void centerView() {
		Rectangle bounds = _gameScroll.getViewport().getViewRect();
		Dimension size = _gameScroll.getViewport().getViewSize();
		int x = (size.width - bounds.width) / 2;
		int y = (size.height - bounds.height) / 2;

		_gameScroll.getViewport().setViewPosition(new Point(x, y));
	}

	public void postInitBuild() {
		_statusColumn.rebuildPluginsColumn();
		_inventoryZone.postInitBuild();
	}

	public void updatePlayerCount() {
		_inventoryZone.updatePlayerCount();
	}
}
