package fr.ensicaen.ecole.easygame.gui;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JComponent;

import fr.ensicaen.ecole.easygame.Game;

/**
 * GUI displaying the game board.
 */
public final class GameZone extends JComponent {

	private static final long serialVersionUID = 1L;

	private Game _game;
	private int _translateX = 0;
	private int _translateY = 0;

	public GameZone(Game game) {
		_game = game;
	}

	@Override
	protected void paintComponent(Graphics g) {
		Dimension viewport = getSize();
		Dimension board = _game.getGameState().getBoard().getDimension();

		setPreferredSize(board);

		if (viewport.width >= board.width) {
			_translateX = viewport.width / 2 - board.width / 2;
		} else {
			_translateX = 0;
		}

		if (viewport.height >= board.height) {
			_translateY = viewport.height / 2 - board.height / 2;
		} else {
			_translateY = 0;
		}

		g.translate(_translateX, _translateY);
		super.paintComponent(g);
		_game.getGameState().draw(g);
		g.translate(-_translateX, -_translateY);
	}

	/**
	 * @return The offset from the left border of the game zone to the left of the board.
	 */
	public int getOffsetX() {
		return _translateX;
	}

	/**
	 * @return The offset from the top border of the game zone to the top of the board.
	 */
	public int getOffsetY() {
		return _translateY;
	}
}