package fr.ensicaen.ecole.easygame.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.state.GameState;

public final class InventoryCellRenderer extends JLabel implements ListCellRenderer<Entity> {

	private static final long serialVersionUID = 1L;
	private Game _game;
	private InventoryZone _inventoryZone;

	public final class InventoryEntityIcon implements Icon {

		private Entity _entity;
		private boolean _selected;

		public InventoryEntityIcon(Entity e, boolean selected) {
			_entity = e;
			_selected = selected;
		}

		@Override
		public int getIconHeight() {
			return _game.getInventoryEntitySize().height;
		}

		@Override
		public int getIconWidth() {
			return _game.getInventoryEntitySize().width;
		}

		@Override
		public void paintIcon(Component c, Graphics g, int x, int u) {
			GameState gs = _game.getGameState();
			Dimension entitySize = _game.getInventoryEntitySize();
			boolean isVisible = gs.getDisplayAllInventory() || _game.getLocalPlayerId() == _inventoryZone.getSelectedPlayer();

			if (_entity == null) {
				return;
			}

			_entity.drawInventory(g, isVisible);

			if (_selected) {
				g.setColor(new Color(0, 0, 0));
				for (int i = 0; i < 4; ++i) {
					g.drawRect(i, i, entitySize.width - 1 - i * 2, entitySize.height - 1 - i * 2);
				}
			}
		}
	}

	public InventoryCellRenderer(Game game, InventoryZone inventoryZone) {
		super();
		_game = game;
		_inventoryZone = inventoryZone;
	}

	@Override
	public JComponent getListCellRendererComponent(JList<? extends Entity> list, Entity value,
												   int index, boolean isSelected, boolean cellHasFocus) {

		setIcon(new InventoryEntityIcon(value, isSelected));

		return this;
	}
}
