package fr.ensicaen.ecole.easygame.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.List;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.state.Inventory;
import fr.ensicaen.ecole.easygame.state.Player;

/**
 * The GUI displaying the inventories.
 */
public final class InventoryZone extends JPanel {

	private static final long serialVersionUID = 1L;
	private Game _game;

	private JPanel _inventoryZone;
	private JPanel _playersZone;
	private JLabel _playerLabel;
	private JList<String> _playersList;
	private JList<Entity> _inventoryList = null;
	private EntityListModel _inventoryModel;
	private int _currentSelectedPlayer = -1;

	public InventoryZone(Game game) {
		_game = game;
		setPreferredSize(new Dimension(0, 120));

		setLayout(new BorderLayout());

		_playersZone = new JPanel();
		_inventoryZone = new JPanel();
		_playerLabel = new JLabel("");

		_inventoryZone.setLayout(new BorderLayout());

		add(_playersZone, BorderLayout.LINE_END);
		add(_inventoryZone, BorderLayout.CENTER);

		_playersZone.setPreferredSize(new Dimension(200, 0));
		_playersZone.setLayout(new BorderLayout());
		_playersZone.add(new JLabel("Joueurs"), BorderLayout.NORTH);
		_playersZone.add(_playerLabel, BorderLayout.SOUTH);
	}

	public void postInitBuild() {
		buildPlayersList();
	}

	private void buildInventoryList() {
		if (_currentSelectedPlayer < 0) {
			return;
		}

		if (_inventoryList == null) {
			_inventoryModel = new EntityListModel();
			_inventoryList = new JList<>(_inventoryModel);
			JScrollPane inventoryScroll = new JScrollPane(_inventoryList);
			InventoryCellRenderer renderer = new InventoryCellRenderer(_game, this);

			_inventoryList.setCellRenderer(renderer);
			_inventoryZone.add(inventoryScroll, BorderLayout.CENTER);
			_inventoryList.setLayout(new FlowLayout());
			_inventoryList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
			_inventoryList.setSelectionModel(new ZeroOrOneSelectionListModel(_inventoryList));
			_inventoryList.setVisibleRowCount(-1);
		}

		{
			final InventoryZone zone = this;

			SwingUtilities.invokeLater(new Runnable() {

				@Override
				public void run() {
					int selected = _inventoryList.getSelectedIndex();

					Inventory inventory = _game.getGameState().getPlayers().get(_currentSelectedPlayer).getInventory();
					inventory.setGui(zone);
					inventory.updateList(_inventoryList, _inventoryModel);

					_inventoryList.revalidate();
					repaint();
					_inventoryList.setSelectedIndex(selected);
				}
			});
		}
	}

	private void buildPlayersList() {
		List<Player> players = _game.getGameState().getPlayers();
		Vector<String> listPlayers = new Vector<>();

		for (Player p : players) {
			listPlayers.add(p.getNickname());
		}

		_playersList = new JList<>(listPlayers);

		JScrollPane listScroller = new JScrollPane(_playersList);
		_playersZone.add(listScroller, BorderLayout.CENTER);

		_playersList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) {
					return;
				}

				int first = e.getFirstIndex();
				int last = e.getLastIndex();

				for (int i = first; i <= last; ++i) {
					if (_playersList.isSelectedIndex(i)) {
						updateInventoryView(i);
						return;
					}
				}
			}
		});

		_playersList.setSelectionModel(new MonoSelectionListModel<>(_playersList));
		updateInventoryView(0);
	}

	public void updatePlayerCount() {
		Player p = _game.getGameState().getCurrentPlayer();
		if (p == null) {
			return;
		}

		int id = p.getId();

		if (_game.getLocalPlayerId() == id) {
			_playerLabel.setText("À vous (" + p.getNickname() + ") de jouer !");
		} else {
			_playerLabel.setText(p.getNickname() + " joue...");
		}
	}

	public void updateInventoryView() {
		if (_game.getGameState().getCurrentPlayer() == null) {
			return;
		}

		updateInventoryView(_game.getGameState().getCurrentPlayer().getId());
	}

	public void updateInventoryView(int id) {
		if (_playersList == null) {
			return;
		}

		_currentSelectedPlayer = id;
		_playersList.setSelectedIndex(id);
		buildInventoryList();
	}

	public int getSelectedPlayer() {
		return _currentSelectedPlayer;
	}

	public Entity getSelectedEntity() {
		return _inventoryList.getSelectedValue();
	}
}
