package fr.ensicaen.ecole.easygame.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.utils.ImageUtils;

public final class StatusColumn extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private static final int WIDTH = 180;

	private Game _game;

	private JPanel _pluginsPanel;

	private JButton _cancelTurnButton;
	private JButton _endTurnButton;
	private JButton _quitButton;

	public StatusColumn(Game game) {
		_game = game;
		_pluginsPanel = new JPanel();

		setPreferredSize(new Dimension(WIDTH, 100));
		setLayout(new BorderLayout());

		Image image = _game.getImage();

		if (image != null) {
			image = ImageUtils.resizeImage(image, 100, 100);

			JLabel label = new JLabel(new ImageIcon(image), JLabel.CENTER);
			add(label, BorderLayout.PAGE_START);
		}

		buildGameButtonsPanel();
		buildPluginsColumn();
	}

	public void rebuildPluginsColumn() {
		remove(_pluginsPanel);
		buildPluginsColumn();
	}

	private void buildPluginsColumn() {
		_pluginsPanel = new JPanel();
		_pluginsPanel.setLayout(new BoxLayout(_pluginsPanel, BoxLayout.PAGE_AXIS));

		int nbPlugins = _game.getNbPlugins();

		for (int i = 0; i < nbPlugins; ++i) {
			JPanel pluginPanel = _game.getPlugin(i).buildUI();
			_pluginsPanel.add(pluginPanel, BorderLayout.NORTH);
		}

		_pluginsPanel.add(new Box.Filler(
				new Dimension(0, 0),
				new Dimension(WIDTH, 4000),
				new Dimension(WIDTH, 4000)));

		add(_pluginsPanel, BorderLayout.CENTER);
	}

	private void buildGameButtonsPanel() {
		JPanel gameButtonsPanel = new JPanel();
		gameButtonsPanel.setLayout(new GridLayout(_game.canCancelTurn() ? 3 : 1, 1));

		_cancelTurnButton = new JButton("Annuler le tour");
		_endTurnButton = new JButton("Fin du tour");
		_quitButton = new JButton("Quitter");

		_cancelTurnButton.addActionListener(this);
		_endTurnButton.addActionListener(this);
		_quitButton.addActionListener(this);

		if (_game.canCancelTurn()) {
			gameButtonsPanel.add(_cancelTurnButton);
			gameButtonsPanel.add(_endTurnButton);
		}

		gameButtonsPanel.add(_quitButton);
		add(gameButtonsPanel, BorderLayout.PAGE_END);
	}

	public void setEndOfTurnEnabled(boolean b) {
		_endTurnButton.setEnabled(b);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(_quitButton)) {
			_game.closeWindow();
		} else if (e.getSource().equals(_endTurnButton)) {
			_game.submitTurn();
		} else if (e.getSource().equals(_cancelTurnButton)) {
			_game.resetTurn();
		}
	}
}
