package fr.ensicaen.ecole.easygame.plugin;

import javax.swing.JPanel;

import fr.ensicaen.ecole.easygame.Game;

/**
 * Implements a plugin.
 * Plugins are used to add interaction between the player and the game other that
 * through placing entities and such.
 * For example it may be used to roll a dice, to display a timer, to add a button to flip a card, etc.
 */
public abstract class Plugin {

	private Game _game;

	/**
	 * Constructs the plugin.
	 *
	 * @param game Current game.
	 */
	public Plugin(Game game) {
		_game = game;
	}

	/**
	 * Function responsible for building the UI and events handler of the plugin.
	 * The panel is then added to the left column of the window.
	 *
	 * @return The built panel.
	 */
	public abstract JPanel buildUI();

	protected final Game getGame() {
		return _game;
	}
}
