package fr.ensicaen.ecole.easygame.state;

import java.awt.Dimension;
import java.awt.Graphics;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Provides the main methods for creating the game board.
 */
public abstract class Board implements Drawable, Serializable {

	private static final long serialVersionUID = 1L;

	private List<Zone> _zones = new ArrayList<>();

	/**
	 * Adds a zone to the board.
	 *
	 * @param z Zone to add.
	 */
	public final void addZone(Zone z) {
		_zones.add(z);
	}

	/**
	 * Accesses the list of zones on the board.
	 *
	 * @return List of zones on the board.
	 */
	public final List<Zone> getZones() {
		return _zones;
	}

	@Override
	public void draw(Graphics g) {
		for (Zone z : _zones) {
			z.draw(g);
		}
	}

	/**
	 * Finds the zone situated at the coordinates the player clicked at.
	 *
	 * @param x
	 * @param y
	 * @return The corresponding zone, or null if none was found.
	 */
	public final Zone getZoneAt(int x, int y) {
		for (Zone z : _zones) {
			if (z.contains(x, y)) {
				return z;
			}
		}

		return null;
	}

	/**
	 * Returns the dimension of the axis-aligned bounding box for the whole board, in pixels.
	 *
	 * @return Dimension of the board.
	 */
	public abstract Dimension getDimension();
}
