package fr.ensicaen.ecole.easygame.state;

import java.awt.Graphics;
import java.io.Serializable;

/**
 * An entity is any object that the players play with and that can be
 * put on the board. For example, a piece or a card.
 */
public abstract class Entity implements Drawable, Serializable {

	private static final long serialVersionUID = 1L;

	private Player _player = null;

	/**
	 * Returns the owner of the entity.
	 *
	 * @return Said player. null if it is not owned by anybody.
	 */
	public final Player getPlayer() {
		return _player;
	}

	/**
	 * Modifies the owner of the entity.
	 * The entity is then added to the player's list of owned entities.
	 *
	 * @param player Owner of the entity.
	 */
	public final void setPlayer(Player player) {
		if (_player == player || player == null) {
			this._player = player;
			return;
		}

		if (_player != null) {
			_player.removeEntity(this);
		}

		player.addEntity(this);
		this._player = player;
	}

	/**
	 * Changes the owner of the entity without changing the players'
	 * list of owned entities.
	 *
	 * @param player The new owner.
	 */
	public final void changePlayer(Player player) {
		this._player = player;
	}

	/**
	 * Method invoked to draw the entity in an inventory.
	 *
	 * @param g Graphics context used for rendering.
	 * @param isVisible Whether or not the entity is visible by the local player.
	 */
	public void drawInventory(Graphics g, boolean isVisible) {
		if (isVisible) {
			draw(g);
		}
	}
}
