package fr.ensicaen.ecole.easygame.state;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Implements a player.
 */
public abstract class Player implements Serializable {

	private static final long serialVersionUID = 1L;

	private List<Entity> _entities = new ArrayList<>();
	private Inventory _inventory = new Inventory(this);
	private int _id;
	private String _nickname;

	public Player(int id, String nickname) {
		_id = id;
		_nickname = nickname;
	}

	/**
	 * Adds an entity to this player's list of owned entities.
	 * Also changes to owner of the entity.
	 *
	 * @param e Entity to be added.
	 */
	public final void addEntity(Entity e) {
		if (_entities.contains(e)) {
			return;
		}

		e.changePlayer(this);
		_entities.add(e);
	}

	/**
	 * Removes an entity from this player's list of owned entities.
	 * ALso changed the owner of the entity.
	 *
	 * @param e Entity to be removed.
	 */
	public final void removeEntity(Entity e) {
		e.changePlayer(null);
		_entities.remove(e);
	}

	/**
	 * @return The entities owned by the player.
	 */
	public final List<Entity> getEntities() {
		return _entities;
	}

	/**
	 * @return The player's ID.
	 */
	public final int getId() {
		return _id;
	}

	/**
	 * @return The player's nickname.
	 */
	public String getNickname() {
		return _nickname;
	}

	/**
	 * @return The player's inventory.
	 */
	public final Inventory getInventory() {
		return _inventory;
	}
}
