package fr.ensicaen.ecole.easygame.state;

import java.awt.Color;
import java.awt.Graphics;

/**
 * An axis-aligned bounding-box.
 */
public class RectangleShape implements Shape {

	private static final long serialVersionUID = 1L;

	private int _w;
	private int _h;
	private boolean _border;

	/**
	 * Constructs a rectangle shape.
	 *
	 * @param w Width of the rectangle.
	 * @param h Height of the rectangle.
	 */
	public RectangleShape(int w, int h) {
		_w = w;
		_h = h;
		_border = true;
	}

	/**
	 * Constructs a rectangle shape.
	 *
	 * @param w Width of the rectangle.
	 * @param h Height of the rectangle.
	 * @param border Displays the border or not
	 */
	public RectangleShape(int w, int h, boolean border) {
		this(w, h);
		_border = border;
	}

	@Override
	public boolean contains(int x, int y) {
		return x >= 0 && x < _w && y >= 0 && y < _h;
	}

	@Override
	public void draw(Graphics g) {
		if (_border) {
			g.setColor(Color.BLACK);
			g.drawRect(0, 0, _w, _h);
		}
	}

	/**
	 * @return Width of the rectangle.
	 */
	public int getW() {
		return _w;
	}

	/**
	 * @return Height of the rectangle.
	 */
	public int getH() {
		return _h;
	}
}
