package fr.ensicaen.ecole.easygame.state;

/**
 * Implements a zone that can contain only one entity.
 */
public class SingleEntityZone extends Zone {

	private static final long serialVersionUID = 1L;

	/**
	 * Constructs a single entity zone.
	 *
	 * @param x x-coordinate of the zone.
	 * @param y y-coordinate of the zone.
	 * @param shape Shape of the zone.
	 */
	public SingleEntityZone(int x, int y, Shape shape) {
		super(x, y, shape);
	}

	/**
	 * Constructs a single entity zone.
	 *
	 * @param x x-coordinate of the zone.
	 * @param y y-coordinate of the zone.
	 * @param shape Shape of the zone.
	 * @param e Entity contained by the zone.
	 */
	public SingleEntityZone(int x, int y, Shape shape, Entity e) {
		super(x, y, shape);
		addEntity(e);
	}

	@Override
	public void addEntity(Entity e) {
		if (!isEmpty()) {
			throw new RuntimeException(this + " is not empty.");
		}

		super.addEntity(e);
	}

	/**
	 * @return The entity on the zone.
	 */
	public Entity getEntity() {
		if (isEmpty()) {
			throw new RuntimeException(this + " is empty.");
		}

		return getEntity(0);
	}
}
