package fr.ensicaen.ecole.easygame.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.jar.JarOutputStream;

public final class FileUtils {

	private FileUtils() {
	}

	public static void copyFileUsingStream(File source, File dest) throws IOException {
		InputStream is = null;
		OutputStream os = null;

		try {
			byte[] buffer = new byte[1024];
			int length;

			is = new FileInputStream(source);
			os = new FileOutputStream(dest);

			while ((length = is.read(buffer)) > 0) {
				os.write(buffer, 0, length);
			}
		} finally {
			if (is != null) {
				is.close();
			}
			if (os != null) {
				os.close();
			}
		}
	}

	public static void updateJarFile(File srcJarFile, File file) throws IOException {
		Path source = srcJarFile.toPath();
		try {
			Files.move(source, source.resolveSibling("Tmp.jar"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		String src = srcJarFile.getPath().substring(0, srcJarFile.getPath().lastIndexOf(File.separator));
		File tempFile = new File(src + "/" + "Tmp.jar");

		byte[] buf = new byte[1024];
		JarInputStream zin = new JarInputStream(new FileInputStream(tempFile));
		JarOutputStream out = new JarOutputStream(new FileOutputStream(srcJarFile));
		JarEntry entry = zin.getNextJarEntry();

		while (entry != null) {
			String name = entry.getName();

			if (file.getName().equals(name)) {
				break;
			}

			out.putNextEntry(new JarEntry(name));

			int len;
			while ((len = zin.read(buf)) > 0) {
				out.write(buf, 0, len);
			}

			entry = zin.getNextJarEntry();
		}

		zin.close();

		InputStream in = new FileInputStream(file);
		out.putNextEntry(new JarEntry(file.getName()));

		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}

		out.closeEntry();
		in.close();

		out.close();
		tempFile.delete();
	}
}
