package fr.ensicaen.ecole.easygame.utils;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;

/**
 * Image-related utility functions.
 */
public final class ImageUtils {

	private ImageUtils() {
	}

	/**
	 * Resizes an image.
	 *
	 * @param image Image to be resized.
	 * @param width New width.
	 * @param height New height.
	 * @return The new image.
	 */
	public static Image resizeImage(Image image, int width, int height) {
		BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics2D = bufferedImage.createGraphics();

		graphics2D.setComposite(AlphaComposite.Src);
		graphics2D.drawImage(image, 0, 0, width, height, null);
		graphics2D.dispose();

		return bufferedImage;
	}

	/**
	 * Opens an image.
	 *
	 * @param filename Path to the image to open.
	 * @return The opened image. null if there was a problem when opening it.
	 */
	public static Image openImage(Class<?> from, String filename) {
		Image img = null;

		try {
			img = ImageIO.read(from.getResourceAsStream(filename));
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		}

		return img;
	}
}
