package fr.ensicaen.ecole.guerredesmoutons;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.Shape;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;

public final class GuerreMoutonsBoard extends Board {

	private static final long serialVersionUID = 1L;

	public static final int W = GuerreMoutonsCard.BOARD_SIZE;

	private int _minx = -1;
	private int _maxx = 1;
	private int _miny = -1;
	private int _maxy = 1;

	GuerreMoutonsBoard() {
		Shape shape = new GrassZoneShape();

		for (int x = _minx; x <= _maxx; ++x) {
			for (int y = _miny; y <= _maxy; ++y) {
				addZone(new SingleEntityZone(x * W, y * W, shape));
			}
		}
	}

	private int countSheep(Board board, int x, int y, List<Point> list, Color color) {
		int count = 0;
		int[][] coords = {{0, -1}, {1, 0}, {0, 1}, {-1, 0}};
		SingleEntityZone z = (SingleEntityZone) board.getZoneAt(x, y);
		GuerreMoutonsCard c = (GuerreMoutonsCard) z.getEntity();

		if (list.contains(new Point(x, y))) {
			return 0;
		}

		list.add(new Point(x, y));

		for (int i = 0; i < 4; ++i) {
			if (c.getSheep(i) == color) {
				count += 1;

				SingleEntityZone zNeighbor = (SingleEntityZone) board.getZoneAt(x + coords[i][0] * W, y + coords[i][1] * W);

				if (zNeighbor == null) {
					continue;
				} else if (zNeighbor.isEmpty()) {
					count -= 10000;
					break;
				}

				GuerreMoutonsCard eNeighbor = (GuerreMoutonsCard) zNeighbor.getEntity();

				if (eNeighbor.getSheep((i + 2) % 4).equals(color)) {
					count += countSheep(board, x + coords[i][0] * W, y + coords[i][1] * W, list, color);
				} else if (eNeighbor.getSheep((i + 2) % 4).equals(Color.GREEN)) {
					List<Point> listWolf = new ArrayList<>();

					if (GuerreMoutonsLogic.searchInForest(board, x + coords[i][0] * W, y + coords[i][1] * W, listWolf, true)) {
						count -= 10000;
						break;
					}
				}
			} else if (c.getSheep(i).equals(Color.GREEN)) {
				List<Point> listWolf = new ArrayList<>();

				if (GuerreMoutonsLogic.searchInForest(board, x, y, listWolf, true)) {
					count -= 10000;
					break;
				}
			}
		}

		return count;
	}

	public HashMap<Color, Integer> count() {
		HashMap<Color, Integer> hm = new HashMap<>();
		Color[] cOrder = {Color.BLUE, Color.BLACK, Color.YELLOW, Color.RED};

		for (Color c : cOrder) {
			hm.put(c, 0);
		}

		for (int i = _minx; i <= _maxx; ++i) {
			for (int j = _miny; j <= _maxy; ++j) {
				SingleEntityZone z = (SingleEntityZone) getZoneAt(i * W, j * W);

				if (z != null && !z.isEmpty()) {
					GuerreMoutonsCard c = (GuerreMoutonsCard) z.getEntity();

					for (int k = 0; k < 4; ++k) {
						if (!c.getSheep(k).equals(Color.GREEN)
								&& !c.getSheep(k).equals(Color.WHITE)) {
							List<Point> visitedSheep = new ArrayList<>();
							int nbSheep = countSheep(this, i * W, j * W, visitedSheep, c.getSheep(k));
							if (nbSheep > hm.get(c.getSheep(k))) {
								hm.put(c.getSheep(k), nbSheep);
							}
						}
					}
				}
			}
		}

		return hm;
	}

	int getWidth() {
		return _maxx - _minx + 1;
	}

	int getHeight() {
		return _maxy - _miny + 1;
	}

	public int getOffsetX() {
		return _minx * W;
	}

	public int getOffsetY() {
		return _miny * W;
	}

	@Override
	public Dimension getDimension() {
		return new Dimension(W * getWidth(), W * getHeight());
	}

	@Override
	public void draw(Graphics g) {
		g.translate(-_minx * W, -_miny * W);
		super.draw(g);
		g.translate(_minx * W, _miny * W);
	}

	public void adaptSize(int zoneX, int zoneY) {
		Shape shape = new GrassZoneShape();

		if (zoneX <= _minx) {
			_minx--;

			for (int i = _miny; i <= _maxy; ++i) {
				addZone(new SingleEntityZone(_minx * W, i * W, shape));
			}
		} else if (zoneX >= _maxx) {
			_maxx++;

			for (int i = _miny; i <= _maxy; ++i) {
				addZone(new SingleEntityZone(_maxx * W, i * W, shape));
			}
		}

		if (zoneY <= _miny) {
			_miny--;

			for (int i = _minx; i <= _maxx; ++i) {
				addZone(new SingleEntityZone(i * W, _miny * W, shape));
			}
		} else if (zoneY >= _maxy) {
			_maxy++;

			for (int i = _minx; i <= _maxx; ++i) {
				addZone(new SingleEntityZone(i * W, _maxy * W, shape));
			}
		}
	}
}
