package fr.ensicaen.ecole.guerredesmoutons;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.util.HashMap;
import java.util.Map;

import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.utils.ImageUtils;

public final class GuerreMoutonsCard extends Entity {

	private static final long serialVersionUID = 1L;

	public static final int INVENTORY_SIZE = 80;
	public static final int BOARD_SIZE = 50;

	private Color[][] _cardinal = new Color[2][4];
	private boolean _isRecto = true;
	private boolean _isWolf = false;
	private boolean _isHunter = false;
	private boolean _isShepherd = false;

	public static class Images {

		public static final Image BACKGROUND = open("background");
		public static final Image FREE = open("free");
		public static final Image HIDDEN = open("hidden");
		public static final Image WOLF = open("wolf");
		public static final Image HUNTER = open("hunter");
		public static final Image FULLVILLAGE = open("village-full");

		public static final Image[] FOREST = {open("forest-north"), open("forest-east"), open("forest-south"), open("forest-west")};
		public static final Image[] VILLAGE = {open("village-north"), open("village-east"), open("village-south"), open("village-west")};

		public static final Image[] REDSHEEP = {open("sheep-red-north"), open("sheep-red-east"), open("sheep-red-south"), open("sheep-red-west")};
		public static final Image[] YELLOWSHEEP = {open("sheep-yellow-north"), open("sheep-yellow-east"), open("sheep-yellow-south"), open("sheep-yellow-west")};
		public static final Image[] BLUESHEEP = {open("sheep-blue-north"), open("sheep-blue-east"), open("sheep-blue-south"), open("sheep-blue-west")};
		public static final Image[] BLACKSHEEP = {open("sheep-black-north"), open("sheep-black-east"), open("sheep-black-south"), open("sheep-black-west")};

		public static final Map<Color, Image[]> SHEEP = initSheep();
		public static final Map<Color, Image> SHEPERD = initSheperd();

		public static final Image[] FENCE = {open("fence-nw"), open("fence-ne"), open("fence-se"), open("fence-sw")};

		private static Image open(String file) {
			return ImageUtils.openImage(GuerreMoutons.class, "/fr/ensicaen/ecole/guerredesmoutons/images/" + file + ".png");
		}

		private static Map<Color, Image> initSheperd() {
			Map<Color, Image> map = new HashMap<>();

			map.put(Color.RED, open("sheperd-red"));
			map.put(Color.YELLOW, open("sheperd-yellow"));
			map.put(Color.BLUE, open("sheperd-blue"));
			map.put(Color.BLACK, open("sheperd-black"));

			return map;
		}

		private static Map<Color, Image[]> initSheep() {
			Map<Color, Image[]> map = new HashMap<>();

			map.put(Color.RED, REDSHEEP);
			map.put(Color.YELLOW, YELLOWSHEEP);
			map.put(Color.BLUE, BLUESHEEP);
			map.put(Color.BLACK, BLACKSHEEP);

			return map;
		}
	}

	GuerreMoutonsCard(Color[] recto, Color[] verso) {
		if (recto.length != 4 || verso.length != 4) {
			return;
		}

		_cardinal[0] = recto;
		_cardinal[1] = verso;

		if (recto == verso) {
			_isShepherd = true;
		}
	}

	GuerreMoutonsCard(boolean isWolf, Color[] verso) {
		Color[] color = {Color.GREEN, Color.GREEN, Color.GREEN, Color.GREEN};

		_cardinal[0] = color;
		_cardinal[1] = verso;
		_isHunter = !isWolf;

		this._isWolf = isWolf;
	}

	void flip() {
		_isRecto = !_isRecto;
	}

	void rotateLeft() {
		Color temp = _isRecto ? _cardinal[0][0] : _cardinal[1][0];

		for (int i = 0; i < 3; ++i) {
			if (_isRecto) {
				_cardinal[0][i] = _cardinal[0][i + 1];
			} else {
				_cardinal[1][i] = _cardinal[1][i + 1];
			}
		}

		if (_isRecto) {
			_cardinal[0][3] = temp;
		} else {
			_cardinal[1][3] = temp;
		}
	}

	void rotateRight() {
		Color temp = _isRecto ? _cardinal[0][3] : _cardinal[1][3];

		for (int i = 3; i > 0; --i) {
			if (_isRecto) {
				_cardinal[0][i] = _cardinal[0][i - 1];
			} else {
				_cardinal[1][i] = _cardinal[1][i - 1];
			}
		}

		if (_isRecto) {
			_cardinal[0][0] = temp;
		} else {
			_cardinal[1][0] = temp;
		}
	}

	Color getSheep(int i) {
		return _isRecto ? _cardinal[0][i] : _cardinal[1][i];
	}

	boolean isWolf() {
		return _isRecto && _isWolf;
	}

	boolean isHunter() {
		return _isRecto && _isHunter;
	}

	@Override
	public void draw(Graphics g) {
		drawCard(g, BOARD_SIZE);
	}

	public void drawInventory(Graphics g, boolean display) {
		if (display) {
			drawCard(g, INVENTORY_SIZE);
		} else {
			g.drawImage(Images.HIDDEN, 0, 0, INVENTORY_SIZE, INVENTORY_SIZE, null);
		}
	}

	private void drawFullForest(Graphics g, int w) {
		for (int i = 0; i < 4; ++i) {
			g.drawImage(Images.FOREST[i], 0, 0, w, w, null);
		}
	}

	private void drawCard(Graphics g, int w) {
		g.drawImage(Images.BACKGROUND, 0, 0, w, w, null);

		if (_isRecto && isWolf()) {
			drawFullForest(g, w);
			g.drawImage(Images.WOLF, 0, 0, w, w, null);
			return;
		}

		if (_isRecto && isHunter()) {
			drawFullForest(g, w);
			g.drawImage(Images.HUNTER, 0, 0, w, w, null);
			return;
		}

		int recto = _isRecto ? 0 : 1;
		boolean fullVillage = true;

		for (int i = 0; i < 4; ++i) {
			if (!_cardinal[recto][i].equals(Color.WHITE)) {
				fullVillage = false;
				break;
			}
		}

		if (fullVillage) {
			g.drawImage(Images.FULLVILLAGE, 0, 0, w, w, null);
			return;
		}

		//Draw forest and village
		for (int i = 0; i < 4; ++i) {
			if (_cardinal[recto][i].equals(Color.GREEN)) {
				g.drawImage(Images.FOREST[i], 0, 0, w, w, null);
			} else if (_cardinal[recto][i].equals(Color.WHITE)) {
				g.drawImage(Images.VILLAGE[i], 0, 0, w, w, null);
			}
		}

		for (int i = 0; i < 4; ++i) {
			if (!_cardinal[recto][i].equals(Color.GREEN) && !_cardinal[recto][i].equals(Color.WHITE)) {
				g.drawImage(Images.SHEEP.get(_cardinal[recto][i])[i], 0, 0, w, w, null);
			}
		}

		if (_isShepherd) {
			g.drawImage(Images.SHEPERD.get(_cardinal[recto][0]), 0, 0, w, w, null);
		}

		//Draw des enclôs
		Color current = _cardinal[recto][3];

		for (int i = 0; i < 4; ++i) {
			if (current != _cardinal[recto][i] && !(_cardinal[recto][i].equals(Color.GREEN) || current.equals(Color.GREEN))) {
				g.drawImage(Images.FENCE[i], 0, 0, w, w, null);
			}

			current = _cardinal[recto][i];
		}
	}
}
