package fr.ensicaen.ecole.guerredesmoutons;

import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.ServerClient;
import fr.ensicaen.ecole.easygame.dialogs.Dialog;
import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.state.Inventory;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;

final class GuerreMoutonsLogic extends MessageReceiver {

	public static boolean searchInForest(Board board, int x, int y, List<Point> list, boolean wolf) {
		int[][] coords = {{0, -1}, {1, 0}, {0, 1}, {-1, 0}};
		int boardW = GuerreMoutonsBoard.W;
		SingleEntityZone z = (SingleEntityZone) board.getZoneAt(x, y);
		GuerreMoutonsCard c = (GuerreMoutonsCard) z.getEntity();
		boolean found = false;

		if (list.contains(new Point(x, y))) {
			return false;
		}

		list.add(new Point(x, y));

		if ((!wolf && c.isHunter()) || (wolf && c.isWolf())) {
			return true;
		}

		for (int i = 0; i < 4; ++i) {
			SingleEntityZone zNeighbor = (SingleEntityZone) board.getZoneAt(x + coords[i][0] * boardW, y + coords[i][1] * boardW);

			if (zNeighbor == null || zNeighbor.isEmpty()) {
				continue;
			}

			if (((GuerreMoutonsCard) zNeighbor.getEntity()).getSheep((i + 2) % 4).equals(Color.GREEN)) {
				found = found || searchInForest(board, x + coords[i][0] * boardW, y + coords[i][1] * boardW, list, wolf);
			}
		}

		return found;
	}

	public static boolean canPlay(Game game, GuerreMoutonsCard c, SingleEntityZone zone) {
		GuerreMoutonsBoard board = (GuerreMoutonsBoard) game.getGameState().getBoard();

		if (game.isEndOfTurn()) {
			return false;
		}

		if (zone == null) {
			return false;
		}

		if (c == null) {
			return false;
		}

		GuerreMoutonsCard card;
		int[][] coords = {{0, -1}, {1, 0}, {0, 1}, {-1, 0}};
		int boardW = GuerreMoutonsBoard.W;
		int nbEmptyCell = 0;

		if (c.isHunter()) {
			if (!zone.isEmpty()) {
				card = (GuerreMoutonsCard) zone.getEntity();

				if (!card.isWolf()) {
					return false;
				}
			} else {
				ArrayList<Point> l = new ArrayList<>();

				for (int i = 0; i < 4; ++i) {
					SingleEntityZone neighborZ = (SingleEntityZone) board.getZoneAt(zone.getX() + coords[i][0] * boardW, zone.getY() + coords[i][1] * boardW);

					if (neighborZ == null || neighborZ.isEmpty()) {
						++nbEmptyCell;

						if (nbEmptyCell == 4) {
							return false;
						}

						continue;
					}

					GuerreMoutonsCard neighbor = (GuerreMoutonsCard) neighborZ.getEntity();

					if (!neighbor.getSheep((i + 2) % 4).equals(Color.GREEN)) {
						return false;
					} else {
						if (searchInForest(board, zone.getX() + coords[i][0] * boardW, zone.getY() + coords[i][1] * boardW, l, true)) {
							return false;
						}
					}
				}
			}
		} else if (!zone.isEmpty()) {
			return false;
		} else if (c.isWolf()) {
			ArrayList<Point> l = new ArrayList<>();

			for (int i = 0; i < 4; ++i) {
				SingleEntityZone neighborZ = (SingleEntityZone) board.getZoneAt(zone.getX() + coords[i][0] * boardW, zone.getY() + coords[i][1] * boardW);

				if (neighborZ == null || neighborZ.isEmpty()) {
					++nbEmptyCell;

					if (nbEmptyCell == 4) {
						return false;
					}

					continue;
				}

				GuerreMoutonsCard neighbor = (GuerreMoutonsCard) neighborZ.getEntity();

				if (!neighbor.getSheep((i + 2) % 4).equals(Color.GREEN)) {
					return false;
				} else {
					if (searchInForest(board, zone.getX() + coords[i][0] * boardW, zone.getY() + coords[i][1] * boardW, l, false)) {
						return false;
					}
				}
			}
		} else {
			for (int i = 0; i < 4; ++i) {
				SingleEntityZone neighborZ = (SingleEntityZone) board.getZoneAt(zone.getX() + coords[i][0] * boardW, zone.getY() + coords[i][1] * boardW);

				if (neighborZ == null || neighborZ.isEmpty()) {
					++nbEmptyCell;

					if (nbEmptyCell == 4) {
						return false;
					}

					continue;
				}

				GuerreMoutonsCard neighbor = (GuerreMoutonsCard) neighborZ.getEntity();

				if (neighbor.getSheep((i + 2) % 4) != c.getSheep(i)) {
					return false;
				}
			}
		}

		return true;
	}

	public static void play(Game game, GuerreMoutonsCard c, SingleEntityZone z) {
		int boardW = GuerreMoutonsBoard.W;
		GuerreMoutonsGameState gs = (GuerreMoutonsGameState) game.getGameState();

		if (c.isHunter()) {
			if (z.getNbEntities() > 0) {
				z.removeEntity(z.getEntity());
			}
		}

		z.addEntity(c);
		gs.getCurrentPlayer().getInventory().removeEntity(c);
		gs.setLastPlayedZone(z);

		int zoneX = z.getX() / boardW;
		int zoneY = z.getY() / boardW;
		((GuerreMoutonsBoard) gs.getBoard()).adaptSize(zoneX, zoneY);

		game.endTurn();
	}

	private int getNbNeighbor(Board board, int x, int y) {
		int nb = 0;
		int[][] coords = {{0, -1}, {1, 0}, {0, 1}, {-1, 0}};
		int boardW = GuerreMoutonsBoard.W;

		for (int i = 0; i < 4; ++i) {
			SingleEntityZone zNeighbor = (SingleEntityZone) board.getZoneAt(x + coords[i][0] * boardW, y + coords[i][1] * boardW);

			if (zNeighbor != null && !zNeighbor.isEmpty()) {
				++nb;
			}
		}

		return nb;
	}

	public void prepare(Game game, PrepareMessage m) {
		GameState gs = game.getGameState();
		gs.setDisplayAllInventory(false);

		Color[] cPlayers = {Color.RED, Color.BLUE, Color.BLACK, Color.YELLOW};
		List<Color> remainingColors = new ArrayList<>();

		Collections.addAll(remainingColors, cPlayers);

		for (ServerClient c : m.getClients()) {
			int n = (int) (Math.random() * remainingColors.size());

			gs.addPlayer(new GuerreMoutonsPlayer(c.getId(), c.getNickname(),
					remainingColors.remove(n)));
		}
	}

	public void mouse(Game game, MouseMessage m) {
		MouseClickEvent e = m.getEvent();
		GuerreMoutonsGameState gs = (GuerreMoutonsGameState) game.getGameState();
		GuerreMoutonsBoard board = (GuerreMoutonsBoard) gs.getBoard();
		int x = e.getX() + board.getOffsetX();
		int y = e.getY() + board.getOffsetY();
		Inventory inv = gs.getCurrentPlayer().getInventory();

		if (inv.isCurrentlyDisplayed()) {
			GuerreMoutonsCard c = (GuerreMoutonsCard) gs.getCurrentPlayer().getInventory().getSelectedEntity();

			if (canPlay(game, c, (SingleEntityZone) board.getZoneAt(x, y))) {
				play(game, c, (SingleEntityZone) board.getZoneAt(x, y));
			}
		}
	}

	private void makeDeck(GuerreMoutonsGameState gs, List<Color[]> cList) {
		// 1st column
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.YELLOW, Color.WHITE}, cList.get(0)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.BLACK, Color.WHITE}, cList.get(1)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.RED, Color.WHITE}, cList.get(2)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.BLUE, Color.WHITE}, cList.get(3)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.RED, Color.WHITE}, cList.get(4)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.YELLOW, Color.WHITE}, cList.get(5)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.BLUE, Color.WHITE}, cList.get(6)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.RED, Color.WHITE}, cList.get(7)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.BLUE, Color.WHITE}, cList.get(8)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.BLACK, Color.WHITE}, cList.get(9)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.BLACK, Color.WHITE}, cList.get(10)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.YELLOW, Color.WHITE}, cList.get(11)));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.BLACK, Color.RED, Color.BLUE, Color.YELLOW}, new Color[]{Color.BLUE, Color.RED, Color.YELLOW, Color.BLACK}));

		// 2nd column
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLUE, Color.YELLOW, Color.GREEN},
				new Color[]{Color.YELLOW, Color.RED, Color.BLACK, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.YELLOW, Color.BLUE, Color.GREEN},
				new Color[]{Color.BLUE, Color.BLACK, Color.RED, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLACK, Color.YELLOW, Color.GREEN},
				new Color[]{Color.YELLOW, Color.BLUE, Color.RED, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.YELLOW, Color.BLACK, Color.GREEN},
				new Color[]{Color.BLUE, Color.BLACK, Color.YELLOW, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.RED, Color.BLACK, Color.GREEN},
				new Color[]{Color.RED, Color.BLACK, Color.BLUE, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLACK, Color.RED, Color.GREEN},
				new Color[]{Color.BLACK, Color.YELLOW, Color.BLUE, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLACK, Color.BLUE, Color.GREEN},
				new Color[]{Color.YELLOW, Color.BLACK, Color.RED, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLUE, Color.BLACK, Color.GREEN},
				new Color[]{Color.BLACK, Color.YELLOW, Color.RED, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.RED, Color.BLUE, Color.GREEN},
				new Color[]{Color.BLACK, Color.RED, Color.YELLOW, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLUE, Color.RED, Color.GREEN},
				new Color[]{Color.RED, Color.YELLOW, Color.BLUE, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.YELLOW, Color.RED, Color.GREEN},
				new Color[]{Color.YELLOW, Color.RED, Color.BLUE, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.RED, Color.YELLOW, Color.GREEN},
				new Color[]{Color.YELLOW, Color.BLACK, Color.BLUE, Color.BLUE}));

		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.BLUE, Color.BLACK, Color.RED, Color.YELLOW},
				new Color[]{Color.RED, Color.BLACK, Color.YELLOW, Color.BLUE}));

		// 3rd column
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.RED, Color.RED},
				new Color[]{Color.YELLOW, Color.BLUE, Color.BLUE, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.RED, Color.RED},
				new Color[]{Color.YELLOW, Color.BLACK, Color.BLACK, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.RED, Color.RED},
				new Color[]{Color.BLACK, Color.BLUE, Color.BLUE, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.BLUE, Color.BLUE},
				new Color[]{Color.RED, Color.BLACK, Color.BLACK, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.BLUE, Color.BLUE},
				new Color[]{Color.BLACK, Color.BLACK, Color.BLUE, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.BLACK, Color.BLACK},
				new Color[]{Color.RED, Color.YELLOW, Color.YELLOW, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.BLACK, Color.BLACK},
				new Color[]{Color.BLUE, Color.BLUE, Color.BLACK, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.YELLOW, Color.YELLOW},
				new Color[]{Color.BLACK, Color.BLACK, Color.BLUE, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.YELLOW, Color.YELLOW},
				new Color[]{Color.RED, Color.BLUE, Color.BLUE, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.RED, Color.RED},
				new Color[]{Color.BLACK, Color.BLUE, Color.YELLOW, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.YELLOW, Color.YELLOW},
				new Color[]{Color.RED, Color.BLUE, Color.BLACK, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.BLACK, Color.BLACK},
				new Color[]{Color.RED, Color.BLUE, Color.YELLOW, Color.YELLOW}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.BLUE, Color.BLUE},
				new Color[]{Color.RED, Color.YELLOW, Color.BLACK, Color.BLACK}));

		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.YELLOW, Color.YELLOW, Color.YELLOW},
				new Color[]{Color.RED, Color.BLACK, Color.YELLOW, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLACK, Color.BLACK, Color.BLACK},
				new Color[]{Color.BLUE, Color.BLACK, Color.YELLOW, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLUE, Color.BLUE, Color.BLUE},
				new Color[]{Color.YELLOW, Color.BLACK, Color.RED, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.RED, Color.RED, Color.RED},
				new Color[]{Color.BLUE, Color.BLACK, Color.RED, Color.YELLOW}));

		// 4th column
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLUE, Color.BLUE, Color.WHITE},
				new Color[]{Color.YELLOW, Color.YELLOW, Color.RED, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.YELLOW, Color.YELLOW, Color.WHITE},
				new Color[]{Color.RED, Color.RED, Color.BLACK, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.BLACK, Color.BLACK, Color.WHITE},
				new Color[]{Color.YELLOW, Color.YELLOW, Color.BLUE, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.RED, Color.RED, Color.WHITE},
				new Color[]{Color.BLACK, Color.BLACK, Color.YELLOW, Color.BLUE}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.YELLOW, Color.YELLOW, Color.GREEN},
				new Color[]{Color.YELLOW, Color.RED, Color.BLUE, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLACK, Color.BLACK, Color.GREEN},
				new Color[]{Color.YELLOW, Color.BLUE, Color.RED, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.RED, Color.RED, Color.GREEN},
				new Color[]{Color.BLUE, Color.RED, Color.YELLOW, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.BLUE, Color.BLUE, Color.GREEN},
				new Color[]{Color.BLUE, Color.YELLOW, Color.RED, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.GREEN, Color.BLACK, Color.GREEN},
				new Color[]{Color.RED, Color.BLUE, Color.YELLOW, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.GREEN, Color.BLUE, Color.GREEN},
				new Color[]{Color.YELLOW, Color.YELLOW, Color.WHITE, Color.RED}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.GREEN, Color.RED, Color.GREEN},
				new Color[]{Color.BLUE, Color.BLUE, Color.WHITE, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.GREEN, Color.GREEN, Color.YELLOW, Color.GREEN},
				new Color[]{Color.BLACK, Color.BLACK, Color.WHITE, Color.RED}));

		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.WHITE, Color.YELLOW, Color.WHITE},
				new Color[]{Color.BLUE, Color.RED, Color.YELLOW, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.WHITE, Color.RED, Color.WHITE},
				new Color[]{Color.RED, Color.BLUE, Color.YELLOW, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.WHITE, Color.BLACK, Color.WHITE},
				new Color[]{Color.BLUE, Color.YELLOW, Color.RED, Color.BLACK}));
		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.WHITE, Color.WHITE, Color.BLUE, Color.WHITE},
				new Color[]{Color.RED, Color.YELLOW, Color.BLUE, Color.BLACK}));

		gs.addCardToDeck(new GuerreMoutonsCard(new Color[]{Color.BLUE, Color.BLACK, Color.YELLOW, Color.RED},
				new Color[]{Color.RED, Color.BLACK, Color.BLUE, Color.YELLOW}));
	}

	public void onStartGame(Game game) {
		GuerreMoutonsGameState gs = (GuerreMoutonsGameState) game.getGameState();
		Board board = gs.getBoard();
		List<Player> players = gs.getPlayers();
		Color[] cPlayers = {Color.RED, Color.BLUE, Color.BLACK, Color.YELLOW};

		for (int i = 0; i < players.size(); ++i) {
			cPlayers[i] = ((GuerreMoutonsPlayer) gs.getPlayers().get(i)).getColor();
			Color cPlayer = cPlayers[i];
			Color[] c = {cPlayer, cPlayer, cPlayer, cPlayer};
			players.get(i).getInventory().addEntity(new GuerreMoutonsCard(c, c));
		}

		Color[] cOrder = {Color.BLUE, Color.BLACK, Color.YELLOW, Color.RED};
		Color[] cOrderJ = {Color.RED, Color.YELLOW, Color.BLUE, Color.BLACK};
		List<Color[]> cList = new ArrayList<>();

		for (int i = 0; i < 4; ++i) {
			for (int j = 0; j < 4; ++j) {
				if (!cOrder[i].equals(cOrderJ[j])) {
					Color[] c = {cOrder[i], cOrder[i], cOrderJ[j], cOrder[i]};
					cList.add(c);
				}
			}
		}

		makeDeck(gs, cList);

		for (int i = 0; i < 4; ++i) {
			gs.addCardToDeck(new GuerreMoutonsCard(false,
					new Color[]{cOrder[i], cOrder[i], cOrder[i], cOrder[i]}));
			gs.addCardToDeck(new GuerreMoutonsCard(true,
					new Color[]{cOrder[i], cOrder[i], cOrder[i], cOrder[i]}));
		}

		//Put 4 cards in player hands
		for (Player p : players) {
			for (int i = 0; i < 4; ++i) {
				p.getInventory().addEntity(gs.drawCardFromDeck());
			}
		}

		//Add the village card on the center of the board
		int center = GuerreMoutonsBoard.W / 2;
		board.getZoneAt(center, center).addEntity(new GuerreMoutonsCard(
				new Color[]{Color.WHITE, Color.WHITE, Color.WHITE, Color.WHITE},
				new Color[]{Color.WHITE, Color.WHITE, Color.WHITE, Color.WHITE}));
	}

	public void onEndGame(Game game) {
		final GuerreMoutonsGameState gs = (GuerreMoutonsGameState) game.getGameState();
		GuerreMoutonsBoard board = (GuerreMoutonsBoard) gs.getBoard();
		final Map<Color, Integer> scores = board.count();

		List<Player> players = new ArrayList<>(gs.getPlayers());
		Collections.sort(players, new Comparator<Player>() {
			@Override
			public int compare(Player p1, Player p2) {
				GuerreMoutonsPlayer gmp1 = (GuerreMoutonsPlayer) p1;
				GuerreMoutonsPlayer gmp2 = (GuerreMoutonsPlayer) p2;
				return (scores.get(gmp2.getColor()) + gs.getPoints(p2))
						- (scores.get(gmp1.getColor()) + gs.getPoints(p1));
			}
		});

		Dialog jd = new Dialog("Fin de partie",
				"Bien joué " + players.get(0).getNickname() + " !");

		for (Player p : players) {
			GuerreMoutonsPlayer gmp = (GuerreMoutonsPlayer) p;
			jd.addLine(p.getNickname() + " (" + gmp.getNameColor() + ")",
					"" + (scores.get(gmp.getColor()) + gs.getPoints(gmp)));
		}

		jd.display();
	}

	public void onAfterTurnSubmit(Game game) {
		GuerreMoutonsGameState gs = (GuerreMoutonsGameState) game.getGameState();
		GuerreMoutonsBoard board = (GuerreMoutonsBoard) gs.getBoard();
		SingleEntityZone zone = gs.getLastPlayedZone();
		Inventory inv = game.getGameState().getCurrentPlayer().getInventory();

		if (zone == null) {
			return;
		}

		for (int i = 0; i < getNbNeighbor(board, zone.getX(), zone.getY()); ++i) {
			GuerreMoutonsCard c = gs.drawCardFromDeck();

			if (c != null) {
				inv.addEntity(c);
			}
		}
	}

	public void onAfterTurn(Game game) {
		boolean isEnd = true;

		for (Player p : game.getGameState().getPlayers()) {
			isEnd = isEnd && ((GuerreMoutonsPlayer) p).hasLeft();
		}

		if (isEnd) {
			game.endGame();
		}
	}

	public void onBeforeTurn(Game game) {
		if (game.getNbPlugins() > 0) {
			((ColorPlugin) game.getPlugin(0)).update();
		}

		((GuerreMoutonsGameState) game.getGameState()).setLastPlayedZone(null);
	}
}
