package fr.ensicaen.ecole.manager;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.List;
import java.util.Random;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.plaf.basic.BasicScrollBarUI;

import fr.ensicaen.ecole.easygame.AIClient;
import fr.ensicaen.ecole.easygame.AILogic;
import fr.ensicaen.ecole.easygame.Client;
import fr.ensicaen.ecole.easygame.HumanClient;
import fr.ensicaen.ecole.easygame.LaunchableGame;
import fr.ensicaen.ecole.easygame.Server;

public final class GameManager extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	public static final String PATH_DIRECTORY = "Games";
	public static final Color LIGHT_GRAY = new Color(255, 255, 255);
	public static final Color GRAY = new Color(150, 150, 150);
	public static final Color BLUE = new Color(102, 161, 202);

	private JButton _host = new JButton("Héberger");
	private JButton _join = new JButton("Rejoindre");

	private JPanel _gameButtonsPanel = new JPanel();
	private JScrollPane _listGameScroll = new JScrollPane(_gameButtonsPanel);
	private JPanel _masterPanel = new JPanel(new BorderLayout());
	private JPanel _lobby = new JPanel();
	private JPanel _panelPlay = new JPanel();
	private JLabel _name = new JLabel();
	private JTextArea _description = new JTextArea();

	private JMenuBar _menuBar = new JMenuBar();
	private JMenu _menu = new JMenu("Aide");
	private JMenu _rules = new JMenu("Règles");
	private JMenuItem _update = new JMenuItem("Mise à jour");
	private JMenuItem _about = new JMenuItem("Au sujet de ...");
	private JMenuItem _rulesWebSite = new JMenuItem("Aller sur le site");
	private int _nbGamesLaunched = 0;

	private InfoGame _currentGame = null;

	public GameManager() {
		setSize(new Dimension(800, 600));
		setMinimumSize(new Dimension(400, 300));
		setTitle("Game Manager");
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		buildInterface();
		updateListGamesInterface();
	}

	public void buildInterface() {
		Font fontLabel = new Font(_name.getFont().getName(), _name.getFont().getStyle(), 30);
		Font fontDescription = new Font(_name.getFont().getName(), _name.getFont().getStyle(), 14);

		_name.setFont(fontLabel);
		add(_masterPanel);

		_gameButtonsPanel.setLayout(new BoxLayout(_gameButtonsPanel, BoxLayout.Y_AXIS));

		_lobby.setLayout(new BorderLayout());
		_lobby.add(_name, BorderLayout.PAGE_START);
		_lobby.add(_description, BorderLayout.CENTER);
		_lobby.setBackground(LIGHT_GRAY);

		_name.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

		_host.addActionListener(this);
		_join.addActionListener(this);

		_name.setHorizontalAlignment(JLabel.CENTER);
		_description.setFont(fontDescription);
		_description.setEditable(false);
		_description.setOpaque(false);
		_description.setLineWrap(true);
		_description.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		_description.setForeground(new Color(50, 50, 50));

		_panelPlay.setBackground(GRAY);
		_panelPlay.add(_join);
		_panelPlay.add(Box.createRigidArea(new Dimension(30, 0)));
		_panelPlay.add(_host);
		_lobby.add(_panelPlay, BorderLayout.PAGE_END);

		_masterPanel.add(_lobby, BorderLayout.CENTER);
		_menuBar.add(_rules);
		_menuBar.add(_menu);

		_menu.add(_update);
		_menu.add(_about);
		_rules.add(_rulesWebSite);
		setJMenuBar(_menuBar);

		_rulesWebSite.addActionListener(this);
		_update.addActionListener(this);
		_about.addActionListener(this);

		_listGameScroll.getVerticalScrollBar().setUnitIncrement(16);
		_listGameScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		_listGameScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		_listGameScroll.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		_listGameScroll.getVerticalScrollBar().setBackground(GRAY);
		_listGameScroll.getVerticalScrollBar().setUI(new BasicScrollBarUI() {
			protected void configureScrollBarColors() {
				thumbDarkShadowColor = Color.GRAY;
				thumbHighlightColor = Color.BLACK;
				thumbLightShadowColor = Color.BLACK;
			}
		});

		_listGameScroll.setPreferredSize(new Dimension(135, 600));

		_masterPanel.add(_listGameScroll, BorderLayout.WEST);
	}

	public void loadGames() {
		File folder = new File(PATH_DIRECTORY);
		File[] gamesJar = folder.listFiles();

		if (gamesJar == null) {
			return;
		}

		for (File f : gamesJar) {
			if (!f.getName().endsWith("jar")) {
				continue;
			}

			JarFile jar;

			try {
				jar = new JarFile(f);
				JarEntry file = jar.getJarEntry("config.xml");

				if (file == null) {
					jar.close();
					continue;
				}

				InputStream is = jar.getInputStream(file);
				XMLConfig xml = new XMLConfig(is);

				Image image = null;
				InputStream isImage = getFile(xml.getImg(), jar);

				if (isImage != null) {
					image = ImageIO.read(isImage);
				}

				InfoGame info = new InfoGame(xml, f.getName(), image);
				GameButton button = new GameButton(info);
				button.setContentAreaFilled(false);
				button.setOpaque(true);
				_gameButtonsPanel.add(button);
				button.addActionListener(this);

				jar.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public InputStream getFile(String filePath, JarFile jar) {
		JarEntry file = jar.getJarEntry(filePath);

		if (file == null) {
			return null;
		}

		InputStream is = null;

		try {
			is = jar.getInputStream(file);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return is;
	}

	private void erreurNullGame() {
		JOptionPane.showMessageDialog(null, "Aucun jeu n'a été sélectionné",
				"Sélectionner un jeu", JOptionPane.INFORMATION_MESSAGE);
	}

	private void erreurNbGames() {
		JOptionPane.showMessageDialog(null, "Un jeu est en cours",
				"Erreur jeu en cours", JOptionPane.INFORMATION_MESSAGE);
	}

	private void updateColorButton() {
		if (_currentGame == null) {
			return;
		}

		for (Component c : _gameButtonsPanel.getComponents()) {
			GameButton button = (GameButton) c;
			button.setBackground(GRAY);
			if (button.getInfo().getName().equals(_currentGame.getName())) {
				button.setBackground(BLUE);
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof GameButton) {
			GameButton gb = (GameButton) e.getSource();
			InfoGame info = gb.getInfo();

			_description.setText(info.getDescription());
			_name.setText(info.getName());
			_name.setAlignmentX((float) (getWidth() / 2 - info.getName().length() * 10) / getWidth());

			_currentGame = info;
			for (Component c : _gameButtonsPanel.getComponents()) {
				c.setBackground(GRAY);
			}

			gb.setBackground(BLUE);
		} else if (e.getSource().equals(_host)) {
			if (_currentGame != null) {
				if (_nbGamesLaunched == 0) {
					HostPanel pl = new HostPanel(this, _currentGame);
					pl.setVisible(true);
				} else {
					erreurNbGames();
				}
			} else {
				erreurNullGame();
			}
		} else if (e.getSource().equals(_join)) {
			if (_currentGame != null) {
				if (_nbGamesLaunched == 0) {
					JoinPanel pl = new JoinPanel(this, _currentGame);
					pl.setVisible(true);
				} else {
					erreurNbGames();
				}
			} else {
				erreurNullGame();
			}
		} else if (e.getSource().equals(_update)) {
			try {
				UpdateDialog update;
				update = new UpdateDialog(this);
				update.setVisible(true);
			} catch (MalformedURLException e2) {
				e2.printStackTrace();
			}
		} else if (e.getSource().equals(_rulesWebSite)) {
			if (_currentGame != null) {
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						new MiniBrowser(_currentGame);
					}
				});
			} else {
				erreurNullGame();
			}
		} else if (e.getSource().equals(_about)) {
			JOptionPane.showMessageDialog(null, "Game Manager:\n" + "Site de téléchargement: http://eg.kidanger.net/");
		}
	}

	private void reinitCurrentGame() {
		_currentGame = null;
		_name.setText("");
		_description.setText("");
	}

	public void updateListGamesInterface() {
		reinitCurrentGame();
	    
		_gameButtonsPanel.removeAll();
		loadGames();
		_gameButtonsPanel.setBackground(GRAY);
		_gameButtonsPanel.setBorder(null);
		_listGameScroll.setViewportView(_gameButtonsPanel);
		_gameButtonsPanel.revalidate();
		updateColorButton();
	}

	public boolean launchGameAsServer(InfoGame info, int port, List<String> playersList) {
		try {
			LaunchableGame lgame = info.getGameInstance();

			int nbLocal = 0;
			JFrame window = null;

			for (String s : playersList) {
				if (!s.equals("IA") && !s.equals("Distant")) {
					nbLocal++;
				}
			}

			// Host
			Server s = new Server(port, lgame, playersList.size(), info.getLoader());
			if (!s.tryBind()) {
				JOptionPane.showMessageDialog(null, "Impossible de lancer le serveur sur ce port.",
						"Erreur réseau", JOptionPane.ERROR_MESSAGE);
				return false;
			}
			s.start();

			if (nbLocal >= 1) {
				_nbGamesLaunched++;
				window = new JFrame();
				window.addWindowListener(new WindowAdapter() {
					@Override
					public void windowClosed(WindowEvent windowEvent) {
						_nbGamesLaunched--;
					}
				});
			}

			Random r = new Random();
			List<Class<? extends AILogic>> ais = lgame.getAvailableAIs();
			for (String player : playersList) {
				if (player.equals("IA")) {
					AILogic ai = ais.get(r.nextInt(ais.size())).newInstance();
					Client c = new AIClient("localhost", port, lgame, info.getLoader(), ai);
					c.start();
				} else if (!player.equals("Distant")) {
					HumanClient c = new HumanClient("localhost", port, lgame, window, info.getLoader(), player);

					c.registerPlugins(lgame);
					c.tryConnect();
					c.start();

					if (nbLocal >= 2) {
						c.setShared();
					}
				}
			}

			if (window != null) {
				window.setTitle(info.getName() + " - Serveur");
				window.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				window.setVisible(true);
				window.setAlwaysOnTop(true);
				window.setAlwaysOnTop(false);
			}
		} catch (IOException | InstantiationException | IllegalAccessException | ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		}
		return true;
	}

	public boolean launchGameAsClient(InfoGame info, String ip, int port, String nickname) {
		try {
			LaunchableGame lgame = info.getGameInstance();

			JFrame window = new JFrame();
			_nbGamesLaunched++;
			window.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosed(WindowEvent windowEvent) {
					_nbGamesLaunched--;
				}
			});

			HumanClient c = new HumanClient(ip, port, lgame, window, info.getLoader(), nickname);

			window.setTitle(info.getName() + " - Connecté à " + ip);

			if (c.tryConnect()) {
				c.registerPlugins(lgame);
				c.start();
				window.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				window.setVisible(true);
				window.setAlwaysOnTop(true);
				window.setAlwaysOnTop(false);
			} else {
				return false;
			}
		} catch (IOException | InstantiationException | IllegalAccessException | ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		}
		return true;
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}

		GameManager gm = new GameManager();
		gm.setVisible(true);
	}
}
