package fr.ensicaen.ecole.manager;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public final class HostPanel extends PlayPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private InfoGame _infogame;
	private JLabel _labelport = new JLabel("Port");
	private JSpinner _port = new JSpinner(new SpinnerNumberModel(4545, 1025, 65534, 1));
	private JPanel _labelField = new JPanel();
	private JPanel _textField = new JPanel();
	private JPanel _north = new JPanel();
	private JPanel _center = new JPanel();
	private JPanel _centerNorth = new JPanel();
	private JPanel _centerCenter = new JPanel();

	private JButton _play = new JButton("Jouer");
	private JSpinner _numberPlayer;
	private JLabel _labelPlayer = new JLabel("Nombre de joueurs");
	private List<ButtonGroup> _players = new ArrayList<ButtonGroup>();
	private GridLayout _gr;

	public HostPanel(GameManager manager, InfoGame game) {
		super(manager);
		_play.addActionListener(this);
		_infogame = game;
		builder();
		setTitle("Héberger partie de " + _infogame.getName());
		pack();
	}

	public void builder() {
		_gr = new GridLayout(_infogame.getMinPlayer(), 3);
		_centerCenter.setLayout(_gr);

		SpinnerNumberModel spinnerEditor = new SpinnerNumberModel(
				_infogame.getMinPlayer(), _infogame.getMinPlayer(),
				_infogame.getMaxPlayer(), 1);
		_numberPlayer = new JSpinner(spinnerEditor);
		JComponent editor = _numberPlayer.getEditor();
		JFormattedTextField tf = ((JSpinner.DefaultEditor) editor).getTextField();
		tf.setColumns(2);
		ChangeListener listener = new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int taille = (int) _numberPlayer.getValue();
				_centerCenter.removeAll();
				buildRadioButtons(taille);
				_centerCenter.revalidate();
			}
		};

		_centerNorth.add(_labelPlayer);
		_centerNorth.add(_numberPlayer);

		_center.setLayout(new BorderLayout());
		_center.add(_centerNorth, BorderLayout.NORTH);
		buildRadioButtons(_infogame.getMinPlayer());
		_numberPlayer.addChangeListener(listener);

		_labelField.setLayout(new BoxLayout(_labelField, BoxLayout.Y_AXIS));
		_textField.setLayout(new BoxLayout(_textField, BoxLayout.Y_AXIS));

		super.getSouthPanel().add(_play);
		_labelport.setLabelFor(_port);
		_port.setPreferredSize(new Dimension(200, 30));
		_labelField.add(_labelport);
		_textField.add(_port);

		_north.add(_labelField);
		_north.add(_textField);

		super.getMainPanel().add(_north, BorderLayout.NORTH);
		super.getMainPanel().add(_center, BorderLayout.CENTER);

		JScrollPane scroll = new JScrollPane(_centerCenter);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scroll.getVerticalScrollBar().setUnitIncrement(16);
		scroll.setPreferredSize(new Dimension(330, 200));
		_center.add(scroll, BorderLayout.SOUTH);
	}

	public void buildRadioButtons(int taille) {
		_gr.setRows(taille);
		_players.clear();

		for (int i = 0; i < taille; i++) {
			ButtonGroup option = new ButtonGroup();
			JRadioButton ia = new JRadioButton("IA");
			final JRadioButton local = new JRadioButton("Local");
			final JTextField nickname = new JTextField(8);
			JRadioButton distant = new JRadioButton("Distant");
			local.setSelected(true);
			final JPanel flow = new JPanel();
			FlowLayout experimentLayout = new FlowLayout();
			flow.setLayout(experimentLayout);
			flow.add(ia);
			flow.add(local);
			flow.add(nickname);
			flow.add(distant);
			_centerCenter.add(flow);

			option.add(ia);
			option.add(local);
			option.add(distant);

			_players.add(option);

			if (!_infogame.hasAI()) {
				ia.setEnabled(false);
			}
			local.putClientProperty("nickname", nickname);
			local.addChangeListener(new ChangeListener() {
				@Override
				public void stateChanged(ChangeEvent e) {
					if (local.isSelected()) {
						nickname.setEnabled(true);
					} else {
						nickname.setEnabled(false);
					}
				}
			});
		}
	}

	public List<String> getPlayers() {
		List<String> players = new LinkedList<String>();
		for (ButtonGroup group : _players) {
			Enumeration<AbstractButton> en = group.getElements();

			while (en.hasMoreElements()) {
				AbstractButton b = en.nextElement();

				if (b.isSelected()) {
					if (b.getText().equals("Local")) {
						JTextField nickname = (JTextField) b.getClientProperty("nickname");
						if (nickname.getText().isEmpty()) {
							nickname.grabFocus();
							return null;
						}
						players.add(nickname.getText());
					} else {
						players.add(b.getText());
					}
				}
			}
		}
		return players;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);

		if (e.getSource().equals(_play)) {
			int port = (int) _port.getValue();
			List<String> players = getPlayers();
			if (players != null) {
				if (super.getManager().launchGameAsServer(_infogame, port, players)) {
					this.setVisible(false);
				}
			}
		}
	}
}
