package fr.ensicaen.ecole.manager;

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import fr.ensicaen.ecole.easygame.LaunchableGame;

public final class InfoGame {

	private String _name;
	private String _description;
	private Image _image;
	private int _minPlayer;
	private int _maxPlayer;
	private String _url;
	private String _jarName;
	private LaunchableGame _launchable;
	private ClassLoader _loader;

	public InfoGame(XMLConfig xml, String jarName, Image image) {
		this._jarName = jarName;

		_image = image;
		_name = xml.getName();
		_description = xml.getDescription();

		if (_description == null) {
			_description = "";
		}

		_minPlayer = Integer.parseInt(xml.getMinPlayer());
		_maxPlayer = Integer.parseInt(xml.getMaxPlayer());
		_url = xml.getURL();
	}

	public String getName() {
		return _name;
	}

	public String getDescription() {
		return _description;
	}

	public int getMinPlayer() {
		return _minPlayer;
	}

	public int getMaxPlayer() {
		return _maxPlayer;
	}

	public boolean hasAI() {
		try {
			LaunchableGame lgame = getGameInstance();
			return lgame.getAvailableAIs().size() > 0;
		} catch (InstantiationException | IllegalAccessException
				| ClassNotFoundException | IOException e) {
			e.printStackTrace();
		}
		return false;
	}

	public String getUrl() {
		return _url;
	}

	public Image getImage() {
		return _image;
	}

	public LaunchableGame getGameInstance() throws InstantiationException, IllegalAccessException, ClassNotFoundException, IOException {
		if (_launchable != null) {
			return _launchable;
		}

		File file = new File(GameManager.PATH_DIRECTORY + '/' + _jarName);

		if (_loader == null) {
			URL[] urls = new URL[]{file.toURI().toURL()};
			_loader = new URLClassLoader(urls, this.getClass().getClassLoader());
		}

		Class<?> launchableClass = null;
		JarFile jar = new JarFile(file.getAbsolutePath());
		Enumeration<JarEntry> entries = jar.entries();

		while (entries.hasMoreElements()) {
			String f = entries.nextElement().toString();

			if (!f.endsWith(".class")) {
				continue;
			}

			String cn = f.substring(0, f.length() - ".class".length());
			cn = cn.replace("/", ".");

			Class<?> c = Class.forName(cn, true, _loader);
			Class<?> p = c.getSuperclass();

			while (p != null && p != LaunchableGame.class) {
				p = p.getSuperclass();
			}

			if (p != null) {
				launchableClass = c;
			}
		}

		jar.close();
		assert launchableClass != null;
		_launchable = (LaunchableGame) launchableClass.newInstance();

		return _launchable;
	}

	public ClassLoader getLoader() {
		return _loader;
	}

}
