package fr.ensicaen.ecole.manager;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public final class UpdateDialog extends JDialog implements ActionListener, MouseListener {

	private static final long serialVersionUID = 1L;

	private XMLConfig _xml;
	private JPanel _mainPanel = new JPanel(new BorderLayout());
	private JButton _updateButton = new JButton("Mettre à jour");
	private JButton _cancelButton = new JButton("Annuler");
	private List<JCheckBox> _listGames = new ArrayList<>();
	private URL _path = null;
	private GameManager _gm;
	private JTextArea _description;
	private double _progress = 0;
	private String[] _jarNames;
	private String[] _descriptions;
	private LoadingDialog _ld;

	public UpdateDialog(GameManager gameManager) throws MalformedURLException {
		_path = new URL("http://eg.kidanger.net/");

		setSize(new Dimension(600, 300));
		setTitle("Mise à jour des jeux");

		_xml = new XMLConfig(getFileUpdate(_path.toString() + "games.xml"));
		_gm = gameManager;
		_description = new JTextArea();
		_description.setLineWrap(true);
		_description.setEditable(false);

		gamesToUpdate();
		_updateButton.addActionListener(this);
		_cancelButton.addActionListener(this);

		JPanel south = new JPanel(new FlowLayout());
		south.add(_cancelButton);
		south.add(_updateButton);
		JLabel title = new JLabel("Mise à jour des jeux", JLabel.CENTER);
		_mainPanel.add(title, BorderLayout.NORTH);
		_mainPanel.add(south, BorderLayout.SOUTH);
		_mainPanel.add(_description, BorderLayout.CENTER);
		add(_mainPanel);
		_descriptions = _xml.getDescriptions();

		setModal(true);
	}

	private String[] getGameNameJar() {
		return _xml.getGamesNamesJars();
	}

	private String getMD5(String path) throws IOException {

		MessageDigest md;
		try {
			md = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return null;
		}
		FileInputStream fis = new FileInputStream(path);

		byte[] dataBytes = new byte[1024];

		int nread;
		while ((nread = fis.read(dataBytes)) != -1) {
			md.update(dataBytes, 0, nread);
		}

		byte[] mdbytes = md.digest();
		StringBuilder sb = new StringBuilder();
		for (byte mdbyte : mdbytes) {
			sb.append(Integer.toString((mdbyte & 0xff) + 0x100, 16).substring(1));
		}
		fis.close();

		return sb.toString();
	}

	public void gamesToUpdate() {
		String[] gamesName = _xml.getNames();
		_jarNames = getGameNameJar();
		String[] versions = _xml.getVersionsGames();
		Box box = Box.createVerticalBox();

		for (int i = 0; i < gamesName.length; i++) {
			JCheckBox gameToUpdate = new JCheckBox(gamesName[i]);
			String path = GameManager.PATH_DIRECTORY + "/" + _jarNames[i] + ".jar";

			try {
				String md5 = getMD5(path);
				if (md5 != null && versions[i].equals(md5)) {
					gameToUpdate.setEnabled(false);
				}
			} catch (IOException ignored) {
			}

			box.add(gameToUpdate);

			gameToUpdate.addActionListener(this);
			gameToUpdate.addMouseListener(this);
			_listGames.add(gameToUpdate);
		}

		_mainPanel.add(box, BorderLayout.WEST);
	}

	private void showInformation(int i) {
		_description.setText(_descriptions[i]);
	}

	private InputStream getFileUpdate(String path) {
		InputStream stream = null;

		try {
			URL xmlUrl = new URL(path);
			URLConnection urlConnection = xmlUrl.openConnection();
			urlConnection.setUseCaches(false);
			urlConnection.connect();
			stream = urlConnection.getInputStream();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return stream;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(_cancelButton)) {
			this.setVisible(false);
		}

		if (e.getSource().equals(_updateButton)) {
			_ld = new LoadingDialog(this);
			_ld.setModal(true);
			_ld.setVisible(true);
			_ld.setAlwaysOnTop(true);
			_ld.setLocationRelativeTo(this);
		}
	}

	public void getUpdate() {
		int i = 0;
		int size = 0;
		for (JCheckBox check : _listGames) {
			if (check.isSelected()) {
				size++;
			}
		}

		for (JCheckBox check : _listGames) {
			if (check.isSelected()) {
				String game = _jarNames[i];

				InputStream is = getFileUpdate(_path + game + ".jar");
				FileOutputStream outputStream;
				File dir = new File(GameManager.PATH_DIRECTORY);

				if (!dir.exists()) {
					dir.mkdirs();
				}

				try {
					outputStream = new FileOutputStream(new File(GameManager.PATH_DIRECTORY + "/" + game + ".jar"));
				} catch (FileNotFoundException e) {
					e.printStackTrace();
					continue;
				}

				int read;
				byte[] bytes = new byte[1024];
				try {
					while ((read = is.read(bytes)) != -1) {
						outputStream.write(bytes, 0, read);
					}
					outputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
				_progress += Math.ceil(100.f / size);
				_ld.majProgress();
			}
			i++;
		}
		_gm.updateListGamesInterface();
	}

	public int getProgress() {
		return (int) _progress;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		int i = 0;

		for (JCheckBox box : _listGames) {
			if (box.equals(e.getSource())) {
				break;
			}
			i++;
		}
		showInformation(i);
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}
}
