package fr.ensicaen.ecole.manager;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public final class XMLConfig {

	private Document _document;

	protected class CustomEntityManager implements EntityResolver {

		@Override
		public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
			URL url = GameManager.class.getResource("/config.dtd");

			if (url != null) {
				return new InputSource(url.toString());
			}

			url = new URL(systemId);
			return new InputSource(url.toString());
		}
	}

	public XMLConfig(InputStream is) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder loader = factory.newDocumentBuilder();

			loader.setEntityResolver(new CustomEntityManager());
			_document = loader.parse(is);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public String getImg() {
		return getValue("img");
	}

	public String getName() {
		return getValue("gameName");
	}

	public String getDescription() {
		return getValue("description");
	}

	public String getMinPlayer() {
		return getValue("minPlayer");
	}

	public String getMaxPlayer() {
		return getValue("maxPlayer");
	}

	public String getURL() {
		return getValue("url");
	}

	public String[] getDescriptions() {
		return getMultipleValue("description");
	}

	public String[] getNames() {
		return getMultipleValue("nameGame");
	}

	public String[] getGamesNamesJars() {
		return getMultipleValue("name");
	}

	public String[] getVersionsGames() {
		return getMultipleValue("version");
	}

	private String[] getMultipleValue(String name) {
		NodeList gamesNameNode = _document.getElementsByTagName(name);
		String[] gameName = new String[gamesNameNode.getLength()];

		for (int i = 0; i < gamesNameNode.getLength(); i++) {
			gameName[i] = gamesNameNode.item(i).getTextContent();
		}

		return gameName;
	}

	private String getValue(String tagname) {
		return _document.getElementsByTagName(tagname).item(0).getTextContent();
	}

	public void setImg(String value) {
		setValue("img", value);
	}

	public void setName(String value) {
		setValue("gameName", value);
	}

	public void setDescription(String value) {
		setValue("description", value);
	}

	public void setUrl(String value) {
		setValue("url", value);
	}

	private void setValue(String tagname, String value) {
		Node n = _document.getElementsByTagName(tagname).item(0);
		n.setTextContent(value);
	}
}
