package fr.ensicaen.ecole.morpion;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.util.List;

import fr.ensicaen.ecole.easygame.AILogic;
import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.LaunchableGame;
import fr.ensicaen.ecole.easygame.MiniManager;
import fr.ensicaen.ecole.easygame.ServerClient;
import fr.ensicaen.ecole.easygame.dialogs.Dialog;
import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.RectangleShape;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;
import fr.ensicaen.ecole.easygame.state.Zone;
import fr.ensicaen.ecole.easygame.utils.ImageUtils;

final class MorpionBoard extends Board {

	private static final long serialVersionUID = 1L;

	MorpionBoard() {
		RectangleShape rect = new RectangleShape(30, 30);

		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				SingleEntityZone z = new SingleEntityZone(i * 30, j * 30, rect);
				addZone(z);
			}
		}
	}

	Zone getAt(int x, int y) {
		return getZones().get(x * 3 + y);
	}

	MorpionPlayer getPlayerAt(int x, int y) {
		SingleEntityZone z = (SingleEntityZone) getAt(x, y);

		if (z.isEmpty()) {
			return null;
		}

		return (MorpionPlayer) z.getEntity().getPlayer();
	}

	@Override
	public Dimension getDimension() {
		return new Dimension(90, 90);
	}
}

final class MorpionPlayer extends Player {

	private static final long serialVersionUID = 1L;

	private Color _color;

	public MorpionPlayer(int id, String nickname, Color color) {
		super(id, nickname);
		_color = color;
	}

	Color getColor() {
		return _color;
	}
}

final class MorpionPiece extends Entity {

	private static final long serialVersionUID = 1L;

	public MorpionPiece(Player p) {
		super.setPlayer(p);
	}

	@Override
	public void draw(Graphics g) {
		g.setColor(((MorpionPlayer) this.getPlayer()).getColor());
		g.fillRect(5, 5, 20, 20);
	}
}

final class MorpionLogic extends MessageReceiver {

	public void mouse(Game game, MouseMessage m) {
		MouseClickEvent e = m.getEvent();
		int x = e.getX();
		int y = e.getY();
		Zone z = game.getGameState().getBoard().getZoneAt(x, y);
		Player p = game.getGameState().getCurrentPlayer();

		if (game.isEndOfTurn()) {
			return;
		}

		if (z != null && z.isEmpty()) {
			MorpionPiece piece = new MorpionPiece(p);
			z.addEntity(piece);
			game.endTurn();
		}
	}

	public void prepare(Game game, PrepareMessage m) {
		for (ServerClient c : m.getClients()) {
			game.getGameState().addPlayer(
					new MorpionPlayer(c.getId(), c.getNickname(),
							c.getId() == 0 ? Color.RED : Color.GREEN));
		}
	}

	private MorpionPlayer getWinner(Game game) {
		MorpionBoard board = (MorpionBoard) game.getGameState().getBoard();

		for (int x = 0; x < 3; x++) {
			if (board.getPlayerAt(x, 0) != null
					&& board.getPlayerAt(x, 0) == board.getPlayerAt(x, 1)
					&& board.getPlayerAt(x, 1) == board.getPlayerAt(x, 2)) {
				return board.getPlayerAt(x, 0);
			}
		}

		for (int y = 0; y < 3; y++) {
			if (board.getPlayerAt(0, y) != null
					&& board.getPlayerAt(0, y) == board.getPlayerAt(1, y)
					&& board.getPlayerAt(1, y) == board.getPlayerAt(2, y)) {
				return board.getPlayerAt(0, y);
			}
		}

		if (board.getPlayerAt(0, 0) != null
				&& board.getPlayerAt(0, 0) == board.getPlayerAt(1, 1)
				&& board.getPlayerAt(1, 1) == board.getPlayerAt(2, 2)) {
			return board.getPlayerAt(0, 0);
		}

		if (board.getPlayerAt(2, 0) != null
				&& board.getPlayerAt(2, 0) == board.getPlayerAt(1, 1)
				&& board.getPlayerAt(1, 1) == board.getPlayerAt(0, 2)) {
			return board.getPlayerAt(2, 0);
		}

		return null;
	}

	public void onAfterTurn(Game game) {
		boolean full = true;
		MorpionBoard board = (MorpionBoard) game.getGameState().getBoard();

		for (Zone z : board.getZones()) {
			if (z.isEmpty()) {
				full = false;
			}
		}

		if (getWinner(game) != null || full) {
			game.endGame();
		}
	}

	public void onEndGame(Game game) {
		MorpionPlayer winner = getWinner(game);
		String gagnant;

		if (winner == null) {
			gagnant = "Personne n'a gagné.";
		} else {
			gagnant = winner.getNickname() + " a gagné !";
		}

		Dialog jd = new Dialog("Fin de partie", gagnant);
		jd.display();
	}
}

public final class Morpion extends LaunchableGame {

	public static void main(String[] args) {
		MiniManager.run("fr.ensicaen.ecole.morpion.Morpion", 2);
	}

	@Override
	public MessageReceiver instantiateLogic() {
		return new MorpionLogic();
	}

	@Override
	public void collectAIClasses(List<Class<? extends AILogic>> l) {
		l.add(MorpionAILogic.class);
	}

	@Override
	public GameState instantiateGameState() {
		return new GameState(new MorpionBoard());
	}

	@Override
	public void registerPlugins(Game game) {
	}

	@Override
	public Dimension getInventoryEntitySize() {
		return new Dimension(1, 1);
	}

	@Override
	public boolean isCancelTurnAllowed() {
		return false;
	}

	@Override
	public Image getImage() {
		return ImageUtils.openImage(Morpion.class, "/fr/ensicaen/ecole/morpion/morpion.jpg");
	}
}
