package fr.ensicaen.ecole.reversi;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.RectangleShape;
import fr.ensicaen.ecole.easygame.state.Shape;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;
import fr.ensicaen.ecole.easygame.state.Zone;

class ReversiBoard extends Board {

	private static final long serialVersionUID = 1L;

	static final int W = 30;

	ReversiBoard() {
		Shape shape = new RectangleShape(W, W);

		for (int x = 0; x < 8; x++) {
			for (int y = 0; y < 8; y++) {
				addZone(new SingleEntityZone(x * W, y * W, shape));
			}
		}
	}

	@Override
	public void draw(Graphics g) {
		g.setColor(Color.GREEN.darker());
		g.fillRect(0, 0, 8 * W, 8 * W);
		super.draw(g);
	}

	boolean isFull() {
		for (Zone z : getZones()) {
			if (z.isEmpty()) {
				return false;
			}
		}

		return true;
	}

	int count(Player p) {
		int i = 0;

		for (Zone z : getZones()) {
			SingleEntityZone zz = (SingleEntityZone) z;

			if (!z.isEmpty() && zz.getEntity().getPlayer() == p) {
				i++;
			}
		}

		return i;
	}

	ReversiPlayer at(int x, int y) {
		SingleEntityZone z = (SingleEntityZone) getZoneAt(x, y);

		if (z != null && !z.isEmpty()) {
			return (ReversiPlayer) z.getEntity().getPlayer();
		}

		return null;
	}

	boolean canPlayAt(Player p, Zone zone) {
		if (!zone.isEmpty()) {
			return false;
		}

		int x = zone.getX() + W / 2;
		int y = zone.getY() + W / 2;

		for (int dx = -1; dx <= 1; dx++) {
			for (int dy = -1; dy <= 1; dy++) {
				if (dx == 0 && dy == 0) {
					continue;
				}

				int xx = x + W * dx;
				int yy = y + W * dy;
				boolean iter = false;

				while (at(xx, yy) != p && at(xx, yy) != null) {
					xx += W * dx;
					yy += W * dy;
					iter = true;
				}

				if (iter && at(xx, yy) == p) {
					return true;
				}
			}
		}

		return false;
	}

	@Override
	public Dimension getDimension() {
		return new Dimension(W * 8, W * 8);
	}
}
