package fr.ensicaen.ecole.reversi;

import java.awt.Color;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.ServerClient;
import fr.ensicaen.ecole.easygame.dialogs.Dialog;
import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.AddEntityToZoneMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;
import fr.ensicaen.ecole.easygame.state.Zone;

class ReversiLogic extends MessageReceiver {

	public void prepare(Game game, PrepareMessage m) {
		for (ServerClient c : m.getClients()) {
			game.getGameState().addPlayer(new ReversiPlayer(c.getId(), c.getNickname(), c.getId() == 0 ? Color.WHITE : Color.BLACK));
		}
	}

	public void mouse(Game game, MouseMessage m) {
		MouseClickEvent e = m.getEvent();
		int x = e.getX();
		int y = e.getY();
		ReversiBoard board = (ReversiBoard) game.getGameState().getBoard();
		Zone z = board.getZoneAt(x, y);
		Player p = game.getGameState().getCurrentPlayer();

		if (game.isEndOfTurn()) {
			return;
		}

		if (z != null && board.canPlayAt(p, z)) {
			z.addEntity(new ReversiPiece(p));
			game.endTurn();
		}
	}

	public void onStartGame(Game game) {
		int w = ReversiBoard.W;
		ReversiGameState gs = (ReversiGameState) game.getGameState();
		Board board = gs.getBoard();

		board.getZoneAt(3 * w, 3 * w).addEntity(new ReversiPiece(gs.getPlayer1()));
		board.getZoneAt(4 * w, 4 * w).addEntity(new ReversiPiece(gs.getPlayer1()));
		board.getZoneAt(3 * w, 4 * w).addEntity(new ReversiPiece(gs.getPlayer2()));
		board.getZoneAt(4 * w, 3 * w).addEntity(new ReversiPiece(gs.getPlayer2()));
	}

	public void onEndGame(Game game) {
		ReversiGameState gs = (ReversiGameState) game.getGameState();
		ReversiBoard board = (ReversiBoard) game.getGameState().getBoard();
		String text;
		int count1 = board.count(gs.getPlayer1());
		int count2 = board.count(gs.getPlayer2());

		if (count1 > count2) {
			text = "Bien joué joueur Blanc";
		} else if (count1 < count2) {
			text = "Bien joué joueur Noir";
		} else {
			text = "Egalité";
		}

		Dialog jd = new Dialog("Fin de partie", text);

		jd.addLine(gs.getPlayer1().getNickname(), "" + count1);
		jd.addLine(gs.getPlayer2().getNickname(), "" + count2);
		jd.display();
	}

	public void afterTurn(Game game) {
		ReversiBoard board = (ReversiBoard) game.getGameState().getBoard();

		if (board.isFull()) {
			game.endGame();
		}
	}

	public void onAddEntityToZone(Game game, AddEntityToZoneMessage m) {
		Player p = m.getEntity().getPlayer();
		int w = ReversiBoard.W;
		ReversiBoard board = (ReversiBoard) game.getGameState().getBoard();

		int x = m.getZone().getX() + w / 2;
		int y = m.getZone().getY() + w / 2;

		for (int dx = -1; dx <= 1; dx++) {
			for (int dy = -1; dy <= 1; dy++) {
				if (dx == 0 && dy == 0) {
					continue;
				}

				int xx = x + w * dx;
				int yy = y + w * dy;
				boolean iter = false;

				while (board.at(xx, yy) != p && board.at(xx, yy) != null) {
					xx += w * dx;
					yy += w * dy;
					iter = true;
				}

				if (iter && board.at(xx, yy) == p) {
					while (xx != x - w * dx || yy != y - w * dy) {
						SingleEntityZone z = (SingleEntityZone) board.getZoneAt(xx, yy);
						z.getEntity().setPlayer(p);
						xx -= w * dx;
						yy -= w * dy;
					}
				}
			}
		}
	}
}
