package fr.ensicaen.ecole.uno;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.util.HashMap;
import java.util.Map;

import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.utils.ImageUtils;

final class UnoCard extends Entity {

	private static final long serialVersionUID = 1L;

	static final Color[] COLOR_LIST = {Color.BLUE, Color.RED, Color.GREEN, Color.YELLOW};
	private static final Map<Color, Color> COLOR_TRANSLATION = initColorTranslation();

	static final int INV_W = 65;
	static final int INV_H = 100;
	static final int BOARD_W = 124;
	static final int BOARD_H = 192;

	private int _nb;
	private Color _c;
	private CardType _type;

	static class Images {

		public static final Image UNO = open("uno");
		public static final Image PLUS_FOUR = open("plus-four");
		public static final Image PLUS_TWO = open("plus-two");
		public static final Image REVERSE = open("reverse");
		public static final Image SKIP = open("skip");
		public static final Image SWITCH_COLOR = open("switch");
		public static final Image[] DIGITS = initDigits();

		private static Image open(String file) {
			return ImageUtils.openImage(Uno.class, "/fr/ensicaen/ecole/uno/images/" + file + ".png");
		}

		private static Image[] initDigits() {
			Image[] images = new Image[10];

			images[0] = open("zero");
			images[1] = open("one");
			images[2] = open("two");
			images[3] = open("three");
			images[4] = open("four");
			images[5] = open("five");
			images[6] = open("six");
			images[7] = open("seven");
			images[8] = open("eight");
			images[9] = open("nine");

			return images;
		}
	}

	private static Map<Color, Color> initColorTranslation() {
		Map<Color, Color> map = new HashMap<>();

		map.put(Color.RED, new Color(255, 54, 54));
		map.put(Color.GREEN, new Color(76, 186, 55));
		map.put(Color.BLUE, new Color(62, 108, 255));
		map.put(Color.YELLOW, new Color(255, 232, 0));

		return map;
	}

	UnoCard(CardType t, Color c) {
		_type = t;
		_c = c;
		_nb = -1;
	}

	UnoCard(int nb, Color c) {
		this(CardType.BASE, c);
		_nb = nb;
	}

	UnoCard(CardType t) {
		this(t, COLOR_LIST[(int) (Math.random() % 4)]);
	}

	CardType getType() {
		return _type;
	}

	Color getColor() {
		return _c;
	}

	int getNb() {
		return _nb;
	}

	void nextColor() {
		for (int i = 0; i < 4; ++i) {
			if (COLOR_LIST[i].equals(_c)) {
				_c = COLOR_LIST[(i + 1) % 4];
				return;
			}
		}
	}

	int getPoints() {
		switch (_type) {
		case BASE:
			return _nb;
		case COLOR:
			return 50;
		case DRAW2:
			return 20;
		case DRAW4:
			return 50;
		case REVERSE:
			return 20;
		case SKIP:
			return 20;
		default:
			return 0;
		}
	}

	@Override
	public void draw(Graphics g) {
		drawCard(g, BOARD_W, BOARD_H);
	}

	private void drawCard(Graphics g, int w, int h) {
		switch (_type) {
		case BASE:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.DIGITS[_nb], 0, 0, w, h, null);
			break;
		case COLOR:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.SWITCH_COLOR, 0, 0, w, h, null);
			break;
		case DRAW2:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.PLUS_TWO, 0, 0, w, h, null);
			break;
		case DRAW4:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.PLUS_FOUR, 0, 0, w, h, null);
			break;
		case REVERSE:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.REVERSE, 0, 0, w, h, null);
			break;
		case SKIP:
			g.setColor(COLOR_TRANSLATION.get(_c));
			g.fillRect(0, 0, w, h);
			g.drawImage(Images.SKIP, 0, 0, w, h, null);
			break;
		default:
			break;
		}
	}

	public void drawInventory(Graphics g, boolean display) {
		if (display) {
			drawCard(g, INV_W, INV_H);
		} else {
			g.drawImage(Images.UNO, 0, 0, INV_W, INV_H, null);
		}
	}
}
