package fr.ensicaen.ecole.uno;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import fr.ensicaen.ecole.easygame.state.AscendingPlayerOrder;
import fr.ensicaen.ecole.easygame.state.DescendingPlayerOrder;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.state.Inventory;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.PlayerOrder;
import fr.ensicaen.ecole.easygame.state.Zone;

final class UnoGameState extends GameState {

	private static final long serialVersionUID = 1L;

	private List<UnoCard> _deck = new ArrayList<>();
	private static PlayerOrder _ascOrder = new AscendingPlayerOrder();
	private static PlayerOrder _dscOrder = new DescendingPlayerOrder();
	private PlayerOrder _current;
	private Random _rand = new Random();
	private int nbRound = 1;
	private boolean isNewRound = false;

	UnoGameState() {
		super(new UnoBoard(), _ascOrder);
		_current = _ascOrder;
		((UnoBoard) getBoard()).setGameState(this);
	}

	void addCardToDeck(UnoCard card) {
		_deck.add(card);
	}

	UnoCard drawCardFromDeck() {
		if (_deck.size() == 0) {
			Zone playZone = ((UnoBoard) getBoard()).getPlayZone();
			while (playZone.getNbEntities() > 1) {
				UnoCard c = (UnoCard) playZone.getEntity(0);
				playZone.removeEntity(c);
				_deck.add(c);
			}
		}

		int id = _rand.nextInt(_deck.size());
		return _deck.get(id);
	}

	void resetPlayerOrder() {
		setPlayerOrder(_ascOrder);
		_current = _ascOrder;
	}

	void reversePlayerOrder() {
		if (_current.getClass().equals(_ascOrder.getClass())) {
			setPlayerOrder(_dscOrder);
			_current = _dscOrder;
		} else {
			setPlayerOrder(_ascOrder);
			_current = _ascOrder;
		}
	}

	Player getNextPlayer() {
		return _current.getNextPlayer(this, getCurrentPlayer());
	}

	private boolean canPlayDRAW4(Inventory inv, Color c) {
		for (int i = 0; i < inv.getEntityCount(); ++i) {
			UnoCard card = (UnoCard) inv.getEntity(i);

			if (!card.getType().equals(CardType.DRAW4)
					&& !card.getType().equals(CardType.COLOR)
					&& card.getColor().equals(c)) {
				return false;
			}
		}

		return true;
	}

	boolean canPlay(Player p, UnoCard c) {
		Zone playZone = ((UnoBoard) getBoard()).getPlayZone();
		UnoCard lastCard = (UnoCard) playZone.getEntity(playZone.getNbEntities() - 1);
		if (lastCard.getColor().equals(c.getColor())
				|| lastCard.getNb() == c.getNb()
				|| c.getType().equals(CardType.DRAW4)
				|| c.getType().equals(CardType.COLOR)) {
			return !(c.getType().equals(CardType.DRAW4) && !canPlayDRAW4(p.getInventory(), lastCard.getColor()));
		}
		return false;
	}

	void play(Player p, UnoCard c) {
		Zone playZone = ((UnoBoard) getBoard()).getPlayZone();
		p.getInventory().removeEntity(c);
		playZone.addEntity(c);
	}
	
	int getNbRound() {
		return nbRound;
	}
	
	void newRound() {
		setNewRound(true);
		++nbRound;
	}
	
	void setNewRound(boolean b) {
		isNewRound = b;
	}
	
	boolean isNewRound() {
		return isNewRound;
	}
}
