package fr.ensicaen.ecole.uno;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.ServerClient;
import fr.ensicaen.ecole.easygame.dialogs.Dialog;
import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.AddEntityToZoneMessage;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.Inventory;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.Zone;

final class UnoLogic extends MessageReceiver {

	private void resetGame(UnoGameState gs) {
		List<Player> players = gs.getPlayers();

		for (Player p : players) {
			Inventory inv = p.getInventory();

			for (int i = 0; i < inv.getEntityCount(); ++i) {
				UnoCard c = (UnoCard) inv.getEntity(i);
				gs.addCardToDeck(c);
				inv.removeEntity(c);
			}
		}

		Zone z = gs.getBoard().getZoneAt(0, 0);
		while (!z.isEmpty()) {
			UnoCard c = (UnoCard) z.getEntity(0);
			gs.addCardToDeck(c);
			z.removeEntity(c);
		}

		for (Player p : gs.getPlayers()) {
			for (int i = 0; i < 7; ++i) {
				p.getInventory().addEntity(gs.drawCardFromDeck());
			}
			((UnoPlayer) p).setCanPlay();
		}

		gs.getBoard().getZoneAt(0, 0).addEntity(gs.drawCardFromDeck());
		gs.resetPlayerOrder();
	}

	public void prepare(Game game, PrepareMessage m) {
		for (ServerClient c : m.getClients()) {
			game.getGameState().addPlayer(new UnoPlayer(c.getId(), c.getNickname()));
		}
	}

	public void onAddEntityToZone(Game game, AddEntityToZoneMessage m) {
		if (m.getZone().getNbEntities() == 1) {
			return; // card is the one from resetGame
		}
		UnoGameState gs = (UnoGameState) game.getGameState();
		UnoPlayer next = (UnoPlayer) gs.getNextPlayer();
		UnoCard c = (UnoCard) m.getEntity();
		switch (c.getType()) {
		case DRAW2:
			for (int i = 0; i < 2; ++i) {
				next.getInventory().addEntity(gs.drawCardFromDeck());
			}

			next.switchCanPlay();
			break;
		case DRAW4:
			for (int i = 0; i < 4; ++i) {
				next.getInventory().addEntity(gs.drawCardFromDeck());
			}

			next.switchCanPlay();
			break;
		case REVERSE:
			gs.reversePlayerOrder();
			break;
		case SKIP:
			next.switchCanPlay();
			break;
		case BASE:
		case COLOR:
		default:
			break;
		}
	}

	public void mouse(Game game, MouseMessage m) {
		MouseClickEvent e = m.getEvent();
		int x = e.getX();
		int y = e.getY();
		UnoGameState gs = (UnoGameState) game.getGameState();
		UnoBoard board = (UnoBoard) game.getGameState().getBoard();
		Zone z = board.getZoneAt(x, y);
		Player p = game.getGameState().getCurrentPlayer();

		if (z != null && !z.isEmpty()) {
			UnoCard c = (UnoCard) p.getInventory().getSelectedEntity();
			if (c != null && gs.canPlay(p, c)) {
				gs.play(p, c);
				game.endTurn();
			}
		}
	}

	public void onStartGame(Game game) {
		UnoGameState gs = (UnoGameState) game.getGameState();

		for (Color c : UnoCard.COLOR_LIST) {
			gs.addCardToDeck(new UnoCard(0, c));

			for (int i = 1; i < 10; ++i) {
				gs.addCardToDeck(new UnoCard(i, c));
				gs.addCardToDeck(new UnoCard(i, c));
			}

			gs.addCardToDeck(new UnoCard(CardType.DRAW2, c));
			gs.addCardToDeck(new UnoCard(CardType.DRAW2, c));
			gs.addCardToDeck(new UnoCard(CardType.REVERSE, c));
			gs.addCardToDeck(new UnoCard(CardType.REVERSE, c));
			gs.addCardToDeck(new UnoCard(CardType.SKIP, c));
			gs.addCardToDeck(new UnoCard(CardType.SKIP, c));
		}

		for (int i = 0; i < 4; ++i) {
			gs.addCardToDeck(new UnoCard(CardType.DRAW4));
			gs.addCardToDeck(new UnoCard(CardType.COLOR));
		}

		resetGame(gs);
	}

	public void onBeforeTurn(Game game) {
		UnoGameState gs = (UnoGameState) game.getGameState();
		UnoPlayer p = (UnoPlayer) gs.getCurrentPlayer();

		if (!p.canPlay()) {
			p.setCanPlay();
			game.endTurn();
		}
		
		if (gs.isNewRound()) {
			resetGame(gs);
			gs.setNewRound(false);
			
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public void afterTurn(Game game) {
		UnoGameState gs = (UnoGameState) game.getGameState();
		boolean isEnd = false;

		for (Player p : gs.getPlayers()) {
			if (!p.getInventory().isEmpty()) {
				continue;
			}
			// add score

			for (Player player : gs.getPlayers()) {
				int score = 0;
				Inventory inv = player.getInventory();

				while (!inv.isEmpty()) {
					UnoCard c = (UnoCard) inv.getEntity(0);
					score += c.getPoints();
					inv.removeEntity(c);
				}

				((UnoPlayer) player).addPoints(score);
			}

			gs.newRound();

			for (Player player : game.getGameState().getPlayers()) {
				if (((UnoPlayer) player).getPoints() >= 500) {
					isEnd = true;
				}
			}
			break;
		}

		if (isEnd) {
			game.endGame();
		}
	}

	public void onEndGame(Game game) {
		UnoGameState gs = (UnoGameState) game.getGameState();
		List<Player> players = new ArrayList<>(gs.getPlayers());

		Collections.sort(players, new Comparator<Player>() {
			@Override
			public int compare(Player p1, Player p2) {
				UnoPlayer up1 = (UnoPlayer) p1;
				UnoPlayer up2 = (UnoPlayer) p2;
				return up1.getPoints() - up2.getPoints();
			}
		});

		Dialog jd = new Dialog("Fin de partie",
				"Bien joué " + players.get(0).getNickname() + " !");

		for (Player p : players) {
			jd.addLine(p.getNickname(), "" + ((UnoPlayer) p).getPoints());
		}

		jd.display();
	}
}
