package net.kidanger.yourchess;

import java.util.List;

import net.kidanger.yourchess.yourchess.coup.Coup;
import net.kidanger.yourchess.yourchess.coup.CoupFactory;
import net.kidanger.yourchess.yourchess.coup.IDeplacement;
import net.kidanger.yourchess.yourchess.coup.Promotion;
import net.kidanger.yourchess.yourchess.Joueur;
import net.kidanger.yourchess.yourchess.Partie;
import net.kidanger.yourchess.yourchess.Position;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.state.Player;
import fr.ensicaen.ecole.easygame.state.Zone;

final class ChessGameState extends GameState {

	private Partie _partie = new Partie();

	public ChessGameState() {
		super(new ChessBoard());
		((ChessBoard) getBoard()).setEchiquier(_partie.getEchiquier());
	}

	public void init() {
		List<Player> players = getPlayers();
		Joueur j1 = ((ChessPlayer) players.get(0)).getJoueur();
		Joueur j2 = ((ChessPlayer) players.get(1)).getJoueur();
		_partie.preparerPartie(j1, j2);
		((ChessBoard) getBoard()).refresh(_partie);
		_partie.commencerPartie();
	}

	public boolean playAt(ChessPlayer player, ChessPiece selected, int x, int y) {
		Coup coup = CoupFactory.create(_partie, selected.getPiece(), Position.get(x, y));

		if (coup == null) {
			return false;
		}

		if (coup instanceof Promotion) {
			Promotion pro = (Promotion) coup;
			while (pro.getPromotion() == null) {
				pro.setPromotion("D");
			}
		}
		if (!_partie.jouerCoup(player.getJoueur(), coup)) {
			return false;
		}
		System.out.println(coup);
		((ChessBoard) getBoard()).refresh(_partie);
		return true;
	}

	public void play(ChessPlayer player, Coup coup) {
		if (!_partie.jouerCoup(player.getJoueur(), coup)) {
			return;
		}
		((ChessBoard) getBoard()).refresh(_partie);
	}

	public boolean isEnded() {
		return _partie.estFinie();
	}

	public ChessPlayer getWinner() {
		Joueur winner = _partie.getGagnant();
		if (winner == null) {
			return null;
		}

		ChessPlayer p1 = (ChessPlayer) getPlayers().get(0);
		if (p1.getJoueur() == winner) {
			return p1;
		}
		return (ChessPlayer) getPlayers().get(1);
	}

	public void unhighlight(ChessPiece piece) {
		piece.highlight(false);
		for (Zone z : getBoard().getZones()) {
			((SelectedZoneShape) z.getShape()).setSelected(false);
		}
	}

	public void highlight(ChessPlayer player, ChessPiece piece) {
		piece.highlight(true);
		List<Coup> plays = _partie.listeDesCoupsJouable(player.getJoueur());
		boolean dark = true;
		// ugly, but who cares?
		for (Coup play : plays) {
			if (!(play instanceof IDeplacement)) {
				continue;
			}
			IDeplacement dep = (IDeplacement) play;
			if (dep.getSource().equals(piece.getPiece().getPosition())) {
				Position dest = dep.getDestination();
				Zone z = getBoard().getZoneAt((dest.getX() - 1) * ChessBoard.W, (8 - dest.getY()) * ChessBoard.W);
				if (z != null) {
					((SelectedZoneShape) z.getShape()).setSelected(true);
					dark = false;
				}
			}
		}
		if (dark) {
			piece.setDarkHighlight();
		}
	}

	public Partie getPartie() {
		return _partie;
	}

	public String getEndOfGameReason() {
		return _partie.getMotifDeFin();
	}
}
