package net.kidanger.yourchess;

import net.kidanger.yourchess.yourchess.Couleur;
import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.ServerClient;
import fr.ensicaen.ecole.easygame.dialogs.Dialog;
import fr.ensicaen.ecole.easygame.events.MouseClickEvent;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.messaging.MouseMessage;
import fr.ensicaen.ecole.easygame.messaging.PrepareMessage;
import fr.ensicaen.ecole.easygame.state.Zone;

final class ChessLogic extends MessageReceiver {

	private ChessPiece _selected;

	public void prepare(Game game, PrepareMessage m) {
		for (ServerClient c : m.getClients()) {
			game.getGameState().addPlayer(new ChessPlayer(c.getId(), c.getNickname(), c.getId() == 0 ? Couleur.blanc : Couleur.noir));
		}
	}

	public void onStartGame(Game game) {
		ChessGameState gs = (ChessGameState) game.getGameState();
		gs.init();
	}

	public void mouse(Game game, MouseMessage m) {
		MouseClickEvent e = m.getEvent();
		int x = e.getX();
		int y = e.getY();
		ChessGameState gs = ((ChessGameState) game.getGameState());
		ChessBoard board = (ChessBoard) gs.getBoard();
		Zone z = board.getZoneAt(x, y);
		ChessPlayer player = (ChessPlayer) gs.getCurrentPlayer();

		if (game.isEndOfTurn()) {
			return;
		}
		if (z == null) {
			return;
		}

		if (_selected == null) {
			if (z.isEmpty()) {
				return;
			}
			_selected = (ChessPiece) z.getEntity(0);
			gs.highlight(player, _selected);
		} else {
			int xx = 1 + z.getX() / ChessBoard.W;
			int yy = 8 - z.getY() / ChessBoard.W;
			if (gs.playAt(player, _selected, xx, yy)) {
				game.endTurn();
			}
			gs.unhighlight(_selected);
			_selected = null;
		}
	}

	public void afterTurn(Game game) {
		ChessGameState gs = ((ChessGameState) game.getGameState());
		if (gs.isEnded()) {
			game.endGame();
		}
	}

	public void onEndGame(Game game) {
		ChessGameState gs = ((ChessGameState) game.getGameState());
		ChessPlayer winner = gs.getWinner();
		String text;
		if (winner != null) {
			text = winner.getNickname() + " a gagné la partie !";
		} else {
			text = "Partie nulle : " + gs.getEndOfGameReason();
		}

		Dialog jd = new Dialog("Fin de partie." , text);
		jd.display();
	}
}
