package net.kidanger.yourchess;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.util.HashMap;
import java.util.Map;

import net.kidanger.yourchess.yourchess.piece.Piece;
import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.utils.ImageUtils;

final class ChessPiece extends Entity {

	private static final class Images {

		private static final Map<String, Image> IMAGES = new HashMap<>();

		private static Image open(String file) {
			return ImageUtils.openImage(ChessPiece.class, "/net/kidanger/yourchess/images/" + file + ".png");
		}

		public static Image get(String name) {
			if (!IMAGES.containsKey(name)) {
				IMAGES.put(name, open(name));
			}
			return IMAGES.get(name);
		}

	}
	private Piece _piece;
	private boolean _dark;
	private boolean _highlight;

	ChessPiece(Piece p) {
		_piece = p;
	}

	@Override
	public void draw(Graphics g) {
		int w = ChessBoard.W;
		g.drawImage(getImage(), 0, 0, w, w, null);
		if (_highlight) {
			if (_dark) {
				g.setColor(new Color(100, 0, 0, 100));
			} else {
				g.setColor(new Color(100, 200, 200, 100));
			}
			g.fillRect(0, 0, w, w);
		}
	}

	private Image getImage() {
		String img = _piece.getNom().toLowerCase();
		if (_piece.getCouleur().isBlanc()) {
			img += "_blanc";
		} else {
			img += "_noir";
		}
		return Images.get(img);
	}

	public int getPlateauX() {
		int x = _piece.getPosition().getX();
		return (x - 1) * ChessBoard.W;
	}

	public int getPlateauY() {
		int y = _piece.getPosition().getY();
		return (8 - y) * ChessBoard.W;
	}

	public Piece getPiece() {
		return _piece;
	}

	@Override
	public String toString() {
		return _piece.toString();
	}

	public void highlight(boolean highlight) {
		_highlight = highlight;
		_dark = false;
	}

	public void setDarkHighlight() {
		_dark = true;
	}
}
