package net.kidanger.yourchess.yourchess;

import java.io.Serializable;

/**
 * Stocke une coordonnée (x appartient à [1, 8] et y appartient à [1, 8])
 * Implémente le Design Pattern Flyweight, qui permet de conservé un petit nombre d'instances pour une classe.
 * Ici, il y a seulement 64 instances de Position dans tout le jeu.
 */
public final class Position implements Serializable {

	private static Position[] positions = new Position[8 * 8];

	static {
		for (int i = 0; i < 64; i++) {
			Position p = new Position(1 + i / 8, 1 + i % 8);
			positions[p.hashCode()] = p;
		}
	}

	private int x;
	private int y;

	private Position(int x, int y) {
		this.x = x;
		this.y = y;
	}

	static public Position get(int x, int y) {
		return positions[(x - 1) * 8 + y - 1];
	}

	public Couleur getCouleur() {
		Couleur c;
		boolean dx = x % 2 == 0;
		boolean dy = y % 2 == 0;
		if (dx == dy) {
			c = Couleur.noir;
		} else {
			c = Couleur.blanc;
		}
		return c;
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	public boolean equals(Position p) {
		return p != null && equals(p.getX(), p.getY());
	}

	public boolean equals(int x, int y) {
		return getX() == x && getY() == y;
	}

	/**
	 * Renvoie une Position à partir d'un chaine de caractère, de format b5 par exemple.
	 */
	static public Position parse(String str) {
		int x = str.charAt(0);
		int y = str.charAt(1);
		x -= 'a' - 1;
		y -= '0';
		if (x < 1 || x > 8 || y < 1 || y > 8) {
			return null;
		}
		return Position.get(x, y);
	}

	public String toString() {
		return (char) (x + 'a' - 1) + "" + y;
	}

	public int hashCode() {
		return (x - 1) * 8 + y - 1;
	}
}
