package net.kidanger.yourchess.yourchess.coup;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Pion;

public class Deplacement extends Coup implements IDeplacement, IPrise {

	static private Pattern pattern = Pattern.compile("^([CTFDR])?([a-h][1-8])([-x])([a-h][1-8])$");

	private Position source;
	private Position destination;
	Piece piece;
	private Piece prise = null;

	public Deplacement(Piece src, Position dst) {
		this.source = src.getPosition();
		this.destination = dst;
		this.piece = src;
	}

	public Deplacement(Piece src, Piece dst) {
		this(src, dst.getPosition());
		this.prise = dst;
	}

	public void jouer(Echiquier e, Couleur c) {
		if (prise != null)
			e.remove(prise);
		e.move(piece, getDestination());
	}

	public void dejouer(Echiquier e, Couleur c) {
		e.unmove(piece, getSource());
		if (prise != null) {
			e.add(prise);
		}
	}

	public Position getSource() {
		return source;
	}

	public Piece getPiece() {
		return piece;
	}

	public Position getDestination() {
		return destination;
	}

	public Piece getPrise() {
		return prise;
	}

	public boolean equals(Coup coup) {
		if (!(coup instanceof Deplacement)) {
			return false;
		}
		Deplacement dep = (Deplacement) coup;
		if (!dep.getSource().equals(getSource()))
			return false;
		return dep.getDestination().equals(getDestination());
	}

	public boolean lowEquals(IDeplacement c) {
		if (!c.getSource().equals(getSource()))
			return false;
		return c.getDestination().equals(getDestination());
	}

	public String notationComplete() {
		if (getPiece() instanceof Pion)
			return "" + getSource() + (getPrise() != null ? "x" : "-") + getDestination();
		char initial = getPiece().getNom().charAt(0);
		return "" + initial + getSource() + (getPrise() != null ? "x" : "-") + getDestination();
	}

	public String notationAbregee(Echiquier e) {
		char initiale = (getPiece() instanceof Pion) ? 0 : getPiece().getNom().charAt(0);
		char prise = getPrise() != null ? 'x' : 0;
		char srcX = 0;
		char srcY = 0;
		String dst = getDestination().toString();

		// Prise par les pions : on note toujours la colonne de départ
		if (getPrise() != null && getPiece() instanceof Pion) {
			srcX = getSource().toString().charAt(0);
		}

		// Détection d'une ambigüité
		// Si une autre piece de même couleur peut atteindre la piece cible, alors
		// - On essaye d'ajouter la colonne (X) si elle est sur la même ligne
		// - Pareil pour la ligne, si elle est sur la même colonne
		for (Piece p : e.getPieces(getPiece().getCouleur())) {
			if (getPiece() == p)
				continue;
			for (Position pos : p.positionsAtteignableEchiquier(e)) {
				if (pos.equals(getDestination())) {
					if (p.getPosition().getX() == getSource().getX()) {
						srcY = getSource().toString().charAt(1);
					}
					if (p.getPosition().getY() == getSource().getY()) {
						srcX = getSource().toString().charAt(0);
					}
				}
			}
		}

		return "" + initiale + srcX + srcY + prise + dst;
	}

	static public Deplacement parseNotationComplete(Echiquier e, Couleur c, String str) {
		// a2-a3
		// Da2-a3
		Matcher m = pattern.matcher(str);

		if (!m.matches()) {
			return null;
		}

		char initiale = 'P';
		boolean hasInitiale = false;
		if (m.group(1) != null) {
			initiale = m.group(1).charAt(0);
			hasInitiale = true;
		}

		Position source = Position.parse(m.group(2));
		char prise = m.group(3).charAt(0);
		Position destination = Position.parse(m.group(4));

		Piece p = e.getPiecePosition(source);
		if (p == null)
			return null;
		if (hasInitiale && p.getNom().charAt(0) != initiale)
			return null;

		Deplacement d = null;
		Piece pieceDestination = e.getPiecePosition(destination);
		if (prise == '-' && pieceDestination == null) {
			d = new Deplacement(p, destination);
		} else if (prise == 'x' && pieceDestination != null && pieceDestination.getCouleur() != c) {
			d = new Deplacement(p, pieceDestination);
		}

		return d;
	}
}
