package net.kidanger.yourchess.yourchess.coup;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Pion;

public class PriseEnPassant extends Coup implements IDeplacement, IPrise {

	static private Pattern pattern = Pattern.compile("^([a-h][1-8])x([a-h][1-8])e.p.$");

	private Pion piece;
	private Piece prise;
	private Position destination;
	private Position source;

	public PriseEnPassant(Pion pion, Piece prise) {
		this.piece = pion;
		this.prise = prise;
		this.destination = Position.get(prise.getPosition().getX(), pion.getPosition().getY() + pion.getDirection());
		this.source = pion.getPosition();
	}

	public Piece getPiece() {
		return piece;
	}

	public Piece getPrise() {
		return prise;
	}

	public Position getSource() {
		return source;
	}

	public Position getDestination() {
		return destination;
	}

	public void jouer(Echiquier e, Couleur c) {
		e.move(piece, getDestination());
		e.remove(prise);
	}

	public void dejouer(Echiquier e, Couleur c) {
		e.add(prise);
		e.unmove(piece, getSource());
	}

	public boolean equals(Coup c) {
		if (!(c instanceof PriseEnPassant))
			return false;
		PriseEnPassant pep = (PriseEnPassant) c;
		if (pep.getPiece() != getPiece())
			return false;
		if (pep.getPrise() != getPrise())
			return false;
		if (!pep.getSource().equals(getSource()))
			return false;
		return pep.getDestination().equals(getDestination());
	}

	public boolean lowEquals(IDeplacement c) {
		if (!c.getSource().equals(getSource()))
			return false;
		return c.getDestination().equals(getDestination());
	}

	public String notationComplete() {
		return getSource() + "x" + getDestination() + "e.p.";
	}

	public String notationAbregee(Echiquier e) {
		// Ajouter "e.p." Empêche presque toutes les ambigüité
		// Pour toutes les empêcher, il suffit d'ajouter le nom de colonne du pion (comme dans toutes les prises par les pions)
		String str = getDestination() + "e.p.";
		if (getPiece() instanceof Pion) {
			str = getSource().toString().charAt(0) + str;
		}
		return str;
	}

	public static PriseEnPassant parseNotationComplete(Echiquier e, Couleur c, String str) {
		// a5xb6e.p.
		Matcher m = pattern.matcher(str);

		if (!m.matches())
			return null;

		Position source = Position.parse(m.group(1));
		Position destination = Position.parse(m.group(2));
		Position intermediaire = Position.get(destination.getX(), source.getY());

		Piece piece = e.getPiecePosition(source);
		Piece pieceIntermediaire = e.getPiecePosition(intermediaire);
		Piece pieceDestination = e.getPiecePosition(destination);

		if (!(piece instanceof Pion))
			return null;
		if (!(pieceIntermediaire instanceof Pion))
			return null;
		if (pieceDestination != null)
			return null;
		if (pieceIntermediaire.getCouleur() == c)
			return null;

		PriseEnPassant pep = new PriseEnPassant((Pion) piece, pieceIntermediaire);

		return pep;
	}
}

