package net.kidanger.yourchess.yourchess.coup;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;
import net.kidanger.yourchess.yourchess.piece.Cavalier;
import net.kidanger.yourchess.yourchess.piece.Dame;
import net.kidanger.yourchess.yourchess.piece.Fou;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Pion;
import net.kidanger.yourchess.yourchess.piece.Tour;

public class Promotion extends Deplacement {

	static private Pattern pattern = Pattern.compile("^([a-h][27])([-x])([a-h][18])[=]?([CTFD])?$");

	private Piece promu = null; // attribuée dans Partie.jouerCoup
	private boolean faked = true; // si "faked" est vrai, cela signifie que le joueur n'a pas choisie la nouvelle piece. Donc dame par defaut.

	public Promotion(Piece src, Position dst) {
		super(src, dst);
	}

	public Promotion(Piece src, Piece dst) {
		super(src, dst);
	}

	public void jouer(Echiquier e, Couleur c) {
		super.jouer(e, c);

		if (promu == null) {
			promu = new Dame(getDestination(), c);
			faked = true;
		}
		e.promotionPiece(piece, promu);
	}

	public void dejouer(Echiquier e, Couleur c) {
		e.promotionPiece(promu, piece); // promotion inverse

		if (faked)
			promu = null;

		super.dejouer(e, c);
	}

	public Pion getPiece() {
		return (Pion) super.getPiece();
	}

	public Piece getPromotion() {
		if (faked)
			return null;
		return promu;
	}

	private void setPromotion(Piece p) {
		promu = p;
		faked = false;
	}

	public void setPromotion(String str) {
		Couleur c = getPiece().getCouleur();
		Piece p = null;
		switch (str) {
		case "D":
			p = new Dame(getDestination(), c);
			break;
		case "F":
			p = new Fou(getDestination(), c);
			break;
		case "C":
			p = new Cavalier(getDestination(), c);
			break;
		case "T":
			p = new Tour(getDestination(), c);
			break;
		default:
			break;
		}
		if (p != null) {
			setPromotion(p);
		}
	}

	public boolean equals(Coup coup) {
		return coup instanceof Promotion && super.equals(coup);
	}

	public String notationComplete() {
		if (promu == null)
			return super.notationComplete() + "=D";
		return super.notationComplete() + "=" + promu.getNom().charAt(0);
	}

	public String notationAbregee(Echiquier e) {
		if (promu == null)
			return super.notationAbregee(e) + "D";
		return super.notationAbregee(e) + promu.getNom().charAt(0);
	}

	public static Promotion parseNotationComplete(Echiquier e, String str) {
		// Pa7-a8=D
		// a7-a8=D
		// a7xb8C

		Matcher m = pattern.matcher(str);

		if (!m.matches())
			return null;

		Position source = Position.parse(m.group(1));
		boolean prise = m.group(2).charAt(0) == 'x';
		Position destination = Position.parse(m.group(3));
		String promotion = m.group(4);

		Piece piece = e.getPiecePosition(source);
		Piece pieceDestination = e.getPiecePosition(destination);

		if (!(piece instanceof Pion))
			return null;
		if (prise && pieceDestination == null)
			return null;
		if (!prise && pieceDestination != null)
			return null;

		Promotion pro;
		if (prise) {
			pro = new Promotion(piece, pieceDestination);
		} else {
			pro = new Promotion(piece, destination);
		}

		if (promotion != null) {
			pro.setPromotion(promotion);
		}

		return pro;
	}
}
