package net.kidanger.yourchess.yourchess.coup;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;
import net.kidanger.yourchess.yourchess.piece.Piece;
import net.kidanger.yourchess.yourchess.piece.Roi;

public class Roque extends Coup implements IDeplacement {

	static private Pattern pattern = Pattern.compile("^O-O(-O)?$");

	static public enum Type {
		PETIT, GRAND
	}

	private Roi roi;
	private Type type;

	public Roque(Roi roi, Type type) {
		this.roi = roi;
		this.type = type;
	}

	public boolean isPetitRoque() {
		return type == Type.PETIT;
	}

	private boolean isGrandRoque() {
		return type == Type.GRAND;
	}

	public Piece getPiece() {
		return roi;
	}

	public Position getSource() {
		int y = roi.getPosition().getY();
		return Position.get(5, y);
	}

	public Position getDestination() {
		int y = roi.getPosition().getY();
		if (isGrandRoque()) {
			return Position.get(3, y);
		}
		return Position.get(7, y);
	}

	public boolean lowEquals(IDeplacement c) {
		if (!c.getPiece().estRoi())
			return false;
		return c.getDestination().equals(getDestination());
	}

	static public Roque parseNotationComplete(Echiquier e, Couleur c, String str) {
		Matcher m = pattern.matcher(str);

		if (!m.matches())
			return null;

		Roque r;

		if (m.group(1) == null) {
			r = new Roque(e.getRoi(c), Type.PETIT);
		} else {
			r = new Roque(e.getRoi(c), Type.GRAND);
		}
		return r;
	}

	public String notationAbregee(Echiquier e) {
		return notationComplete();
	}

	public void jouer(Echiquier e, Couleur c) {
		e.realiserRoque(this);
	}

	public void dejouer(Echiquier e, Couleur c) {
		e.derealiserRoque(this);
	}

	public String notationComplete() {
		if (isGrandRoque()) {
			return "O-O-O";
		}
		return "O-O";
	}

	public boolean equals(Coup coup) {
		if (!coup.isRoque())
			return false;
		return ((Roque) coup).isGrandRoque() == isGrandRoque();
	}
}
