package net.kidanger.yourchess.yourchess.ordinateur;

import java.util.List;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.coup.Coup;

public class OrdinateurIntelligent extends Ordinateur {

	private static long timeMax = 5000; // ms

	private Evaluateur evaluateur;

	public OrdinateurIntelligent(Couleur couleur) {
		super("Intelligent", couleur);
		this.evaluateur = new EvaluateurFinDePartie(new EvaluateurPrise(new EvaluateurCoefficient()));
	}

	public OrdinateurIntelligent(Couleur couleur, Evaluateur e) {
		super("Intelligent", couleur);
		this.evaluateur = e;
	}

	public Coup jouerCoup() {
		return approfondissement();
	}

	private Coup approfondissement() {
		long time = System.currentTimeMillis();

		Coup coup = negamax_init(time, 1, couleur);

		int d;
		// Profondeur 5 : 77499 secondes, en début de jeu
		for (d = 3; d <= 4; d++) {
			Coup tmp = negamax_init(time, d, couleur);
			if (time + timeMax < System.currentTimeMillis()) {
				break;
			}
			coup = tmp;
		}
		d--;
		System.out.println("[intel] en " + coup + ", depth = " + d + ", actual = " + evaluateur.evalue(partie, couleur) + ", time = " + (System.currentTimeMillis() - time));
		return coup;
	}

	private Coup negamax_init(long time, int depth, Couleur c) {
		List<Coup> coups = partie.listeDesCoupsCouleur(c);

		if (coups.isEmpty())
			return null;

		int alpha = -100000;
		int beta = -alpha;

		Coup best = null;

		for (Coup coup : coups) {
			partie.testJouerCoup(c, coup);

			int val = -negamax(time, depth - 1, c.other(), -beta, -alpha);

			if (val > alpha) {
				alpha = val;
				best = coup;
			}

			partie.testDejouerCoup(c, coup);

			if (val >= beta) {
				break;
			}
		}

		return best;
	}

	private int negamax(long time, int depth, Couleur c, int alpha, int beta) {
		if (time + timeMax < System.currentTimeMillis()) {
			return 0;
		}

		List<Coup> coups = partie.listeDesCoupsCouleur(c);
		if (depth == 0 || coups.size() == 0) {
			return evaluateur.evalue(partie, c);
		}

		for (Coup coup : coups) {

			partie.testJouerCoup(c, coup);

			int val = -negamax(time, depth - 1, c.other(), -beta, -alpha);

			if (val >= alpha) {
				alpha = val;
			}

			partie.testDejouerCoup(c, coup);

			if (val >= beta) {
				break;
			}
		}

		return alpha;
	}

	public String getNom() {
		return "Alta (" + evaluateur + ")";
	}

	public String toString() {
		return getNom();
	}

	static public void setTimeMax(long l) {
		timeMax = l;
	}
}
