package net.kidanger.yourchess.yourchess.piece;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;

abstract public class Piece implements Comparable<Piece>, Serializable {

	private String nom;
	Couleur couleur;
	Position position;

	private int mouvements = 0;
	private boolean displayed = true;
	private boolean alive = false;

	private List<Position> tmp = null;
	private long ehash = 0;

	Piece(Position position, Couleur couleur) {
		this.nom = this.getClass().getSimpleName();
		this.position = position;
		this.couleur = couleur;
	}

	abstract List<Position> getPossiblePositions(Echiquier e, boolean checkOthers);

	abstract public int getPoints();

	abstract public char toSmallString();

	public List<Position> positionsAtteignable() {
		return getPossiblePositions(null, false);
	}

	public List<Position> positionsAtteignableEchiquier(Echiquier e) {
		if (e.hashCode2() == ehash && tmp != null)
			return tmp;

		List<Position> list = getPossiblePositions(e, true);
		tmp = list;
		ehash = e.hashCode2();
		return list;
	}

	public List<Piece> priseDePieces(Echiquier e) {
		List<Piece> liste = new LinkedList<>();
		for (Position p : positionsAtteignableEchiquier(e)) {
			Piece piece = e.getPiecePosition(p);
			if (piece != null && !isFriendly(piece)) {
				liste.add(piece);
			}
		}
		return liste;
	}

	boolean isFriendly(Piece p) {
		return this.couleur == p.couleur;
	}

	public void onMove(Position dst) {
		mouvements += 1;
		position = dst;
	}

	public void onUnmove(Position dst) {
		mouvements -= 1;
		position = dst;
	}

	public boolean hasMoved() {
		return mouvements != 0;
	}

	public boolean isAlive() {
		return alive;
	}

	public void setAlive(boolean a) {
		this.alive = a;
	}

	public Position getPosition() {
		return position;
	}

	public Couleur getCouleur() {
		return couleur;
	}

	public boolean estRoi() {
		return false;
	}

	public String getNom() {
		return nom;
	}

	public String toString() {
		return nom + " " + couleur;
	}

	public boolean isDisplayed() {
		return displayed;
	}

	public void setDisplay(boolean d) {
		this.displayed = d;
	}

	public int compareTo(Piece p) {
		return p.getPoints() - this.getPoints();
	}
}
