package net.kidanger.yourchess.yourchess.piece;

import java.util.LinkedList;
import java.util.List;

import net.kidanger.yourchess.yourchess.Couleur;
import net.kidanger.yourchess.yourchess.Echiquier;
import net.kidanger.yourchess.yourchess.Position;

public class Pion extends Piece {

	private int direction; // 1 : vers le haut, -1 : vers le bas

	public Pion(Position position, Couleur couleur) {
		super(position, couleur);
		if (couleur.isBlanc()) {
			direction = 1;
		} else {
			direction = -1;
		}
	}

	protected void add(List<Position> l, Echiquier e, int x, int y) {
		if (e == null) {
			l.add(Position.get(x, y));
			return;
		}
		Piece p = e.getPiecePosition(x, y);
		if (p != null) {
			if (!isFriendly(p)) {
				l.add(Position.get(x, y));
			}
		} else {
			l.add(Position.get(x, y));
		}
	}

	public List<Position> getPossiblePositions(Echiquier e, boolean checkOthers) {
		List<Position> liste = new LinkedList<>();
		int tx = position.getX();
		int ty = position.getY();
		Piece p;

		if (ty == 8 || ty == 1) {
			return liste; // should not be possible
		}

		// note : on peut pas bouffer par devant
		if (!checkOthers || e.getPiecePosition(tx, ty + direction) == null) {
			liste.add(Position.get(tx, ty + direction));
		}

		if (!hasMoved()) {
			if (!checkOthers || (e.getPiecePosition(tx, ty + direction * 2) == null && e.getPiecePosition(tx, ty + direction) == null)) {
				liste.add(Position.get(tx, ty + direction * 2));
			}
		}

		if (checkOthers) {
			if (tx < 8) {
				p = e.getPiecePosition(tx + 1, ty + direction);
				if (p != null && !isFriendly(p)) {
					liste.add(p.getPosition());
				}
			}

			if (tx > 1) {
				p = e.getPiecePosition(tx - 1, ty + direction);
				if (p != null && !isFriendly(p)) {
					liste.add(p.getPosition());
				}
			}
		}
		return liste;
	}

	public int getPoints() {
		return 1;
	}

	public int getDirection() {
		return direction;
	}

	public char toSmallString() {
		if (couleur.isBlanc())
			return '♟';
		return '♙';
	}
}
