package fr.ensicaen.ecole.easygame.testRelation;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import fr.ensicaen.ecole.easygame.state.Board;
import fr.ensicaen.ecole.easygame.state.RectangleShape;
import fr.ensicaen.ecole.easygame.state.Shape;
import fr.ensicaen.ecole.easygame.state.SingleEntityZone;
import fr.ensicaen.ecole.easygame.state.Zone;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 * Class to test the links between the Board and the zones it contains.
 */
public final class BoardZonesTest {

	private Board _board;
	private List<Zone> _zones = new ArrayList<>();

	public final class DummyBoard extends Board {

		private static final long serialVersionUID = 1L;

		@Override
		public Dimension getDimension() {
			return new Dimension(10, 10);
		}
	}

	@Before
	/**
	 * Initialization of different zones and of an entity
	 */
	public void setup() {
		_board = new DummyBoard();
	}

	@Test
	/**
	 * Test if when we create a new single entity zone, it's not null, and when we add this zone, the board zone list contain the element and is not null
	 */
	public void zoneEntityTestAddToBoardTest() {
		Zone z1 = new SingleEntityZone(0, 0, null);

		assertNotNull(z1);
		_board.addZone(z1);
		assertTrue(_board.getZones().contains(z1));
		assertTrue(_board.getZones().size() == 1);
	}

	/**
	 * Test the adding of 1 Zone on the Board
	 */
	@Test
	public void boardAdd1Zone() {
		Shape s = new RectangleShape(10, 10);
		Zone z1 = new Zone(0, 0, s);

		_board.addZone(z1);
		_zones.add(z1);

		assertEquals(_board.getZoneAt(0, 0), z1);

		assertEquals(_board.getZones().size(), 1);
		assertEquals(_board.getZones(), _zones);
	}

	/**
	 * Test the adding of more than 1 Zone on the Board
	 */
	@Test
	public void boardAddMultipleZones() {
		Shape s = new RectangleShape(10, 10);

		Zone z1 = new Zone(0, 0, s);
		Zone z2 = new Zone(10, 10, s);
		Zone z3 = new Zone(20, 20, s);

		_board.addZone(z1);
		_board.addZone(z2);
		_board.addZone(z3);

		_zones.add(z1);
		_zones.add(z2);
		_zones.add(z3);

		assertEquals(_board.getZones().size(), 3);

		assertEquals(_board.getZoneAt(0, 0), z1);
		assertEquals(_board.getZoneAt(10, 10), z2);
		assertEquals(_board.getZoneAt(20, 20), z3);

		assertEquals(_board.getZones(), _zones);
	}
}
