package fr.ensicaen.ecole.easygame.testRelation;

import java.awt.Dimension;
import java.awt.Image;
import java.util.List;

import fr.ensicaen.ecole.easygame.AILogic;
import fr.ensicaen.ecole.easygame.Game;
import fr.ensicaen.ecole.easygame.LaunchableGame;
import fr.ensicaen.ecole.easygame.messaging.MessageReceiver;
import fr.ensicaen.ecole.easygame.state.Entity;
import fr.ensicaen.ecole.easygame.state.GameState;
import fr.ensicaen.ecole.easygame.state.Zone;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 * Test class for the link between Zone and Entity tests : class Zone is not
 * abstract finally.
 */
public final class ZoneEntityTest {

	private Zone _zone1;
	private Zone _zone2;

	class DummyLaunchableGame extends LaunchableGame {

		@Override
		public MessageReceiver instantiateLogic() {
			return null;
		}

		@Override
		public GameState instantiateGameState() {
			return null;
		}

		@Override
		public void registerPlugins(Game game) {
		}

		@Override
		protected void collectAIClasses(List<Class<? extends AILogic>> l) {
		}

		@Override
		public Dimension getInventoryEntitySize() {
			return new Dimension(1, 1);
		}

		@Override
		public boolean isCancelTurnAllowed() {
			return false;
		}

		@Override
		public Image getImage() {
			return null;
		}
	}

	public ZoneEntityTest() {
		MessageReceiver messageReceiver = new MessageReceiver() {
		};
		messageReceiver.link(new Game(new DummyLaunchableGame(), false, null));
	}

	@Before
	/**
	 * Initialization of different zones and of an entity.
	 */
	public void setup() {
		_zone1 = new Zone(0, 0, null);
		_zone2 = new Zone(0, 0, null);
	}

	@Test
	/**
	 * Test if the zone is null at the beginning, and after adding if it is not null
	 * and contains an entity which has the same Zone.
	 */
	public void zoneAddEntity() {
		Entity entity1;
		entity1 = new TestEntity();

		_zone1.addEntity(entity1);
		assertFalse(_zone1.isEmpty());
		assertSame(_zone1.getEntity(0), entity1);
	}

	@Test
	/**
	 * Test if the zone adding is correct for the entity and that we can delete entity after.
	 */
	public void zoneRemoveEntity() {
		Entity entity1;
		entity1 = new TestEntity();

		_zone1.addEntity(entity1);

		_zone1.removeEntity(entity1);
		assertTrue(_zone1.isEmpty());
	}

	@Test
	/**
	 * Test if adding to two different zone works.
	 */
	public void entityChangeZone() {
		Entity entity1, entity2;
		entity1 = new TestEntity();
		entity2 = new TestEntity();

		_zone1.addEntity(entity1);
		assertNotNull(_zone1.getEntity(0));
		assertEquals(_zone1.getEntity(0), entity1);

		_zone2.addEntity(entity2);
		assertNotNull(_zone2.getEntity(0));
		assertEquals(_zone2.getEntity(0), entity2);
	}

	@Test
	/**
	 * Test if entities are removed in order for a zone.
	 */
	public void zoneMultipleEntities() {
		Entity entity1, entity2, entity3;

		entity1 = new TestEntity();
		entity2 = new TestEntity();
		entity3 = new TestEntity();

		_zone1.addEntity(entity1);
		_zone1.addEntity(entity2);
		_zone1.addEntity(entity3);

		assertSame(_zone1.getEntity(0), entity1);
		assertSame(_zone1.getEntity(1), entity2);
		assertSame(_zone1.getEntity(2), entity3);

		_zone1.removeEntity(entity3);
		_zone1.removeEntity(entity2);
		_zone1.removeEntity(entity1);

		assertTrue(_zone1.isEmpty());
	}
}
